@@ -1,5 +1,31 @@
-"""
-Simplex method for solving linear programming problems
+"""Simplex method for  linear programming
+
+The *simplex* method uses a traditional, full-tableau implementation of
+Dantzig's simplex algorithm [1]_, [2]_ (*not* the Nelder-Mead simplex).
+This algorithm is included for backwards compatibility and educational
+purposes.
+
+    .. versionadded:: 0.15.0
+
+Warnings
+--------
+
+The simplex method may encounter numerical difficulties when pivot
+values are close to the specified tolerance. If encountered try
+remove any redundant constraints, change the pivot strategy to Bland's
+rule or increase the tolerance value.
+
+Alternatively, more robust methods maybe be used. See
+:ref:`'interior-point' <optimize.linprog-interior-point>` and
+:ref:`'revised simplex' <optimize.linprog-revised_simplex>`.
+
+References
+----------
+.. [1] Dantzig, George B., Linear programming and extensions. Rand
+       Corporation Research Study Princeton Univ. Press, Princeton, NJ,
+       1963
+.. [2] Hillier, S.H. and Lieberman, G.J. (1995), "Introduction to
+       Mathematical Programming", McGraw-Hill, Chapter 4.
 """
 
 import numpy as np
@@ -7,15 +33,16 @@ from warnings import warn
 from .optimize import OptimizeResult, OptimizeWarning, _check_unknown_options
 from ._linprog_util import _postsolve
 
-def _pivot_col(T, tol=1.0E-12, bland=False):
+
+def _pivot_col(T, tol=1e-9, bland=False):
     """
     Given a linear programming simplex tableau, determine the column
     of the variable to enter the basis.
 
     Parameters
     ----------
-    T : 2D array
-        A 2D array representing the simplex tableau, T, corresponding to the
+    T : 2-D array
+        A 2-D array representing the simplex tableau, T, corresponding to the
         linear programming problem. It should have the form:
 
         [[A[0, 0], A[0, 1], ..., A[0, n_total], b[0]],
@@ -63,19 +90,20 @@ def _pivot_col(T, tol=1.0E-12, bland=False):
     if ma.count() == 0:
         return False, np.nan
     if bland:
-        return True, np.nonzero(ma.mask == False)[0][0]
+        # ma.mask is sometimes 0d
+        return True, np.nonzero(np.logical_not(np.atleast_1d(ma.mask)))[0][0]
     return True, np.ma.nonzero(ma == ma.min())[0][0]
 
 
-def _pivot_row(T, basis, pivcol, phase, tol=1.0E-12, bland=False):
+def _pivot_row(T, basis, pivcol, phase, tol=1e-9, bland=False):
     """
     Given a linear programming simplex tableau, determine the row for the
     pivot operation.
 
     Parameters
     ----------
-    T : 2D array
-        A 2D array representing the simplex tableau, T, corresponding to the
+    T : 2-D array
+        A 2-D array representing the simplex tableau, T, corresponding to the
         linear programming problem. It should have the form:
 
         [[A[0, 0], A[0, 1], ..., A[0, n_total], b[0]],
@@ -138,7 +166,7 @@ def _pivot_row(T, basis, pivcol, phase, tol=1.0E-12, bland=False):
     return True, min_rows[0]
 
 
-def _apply_pivot(T, basis, pivrow, pivcol, tol=1e-12):
+def _apply_pivot(T, basis, pivrow, pivcol, tol=1e-9):
     """
     Pivot the simplex tableau inplace on the element given by (pivrow, pivol).
     The entering variable corresponds to the column given by pivcol forcing
@@ -146,8 +174,8 @@ def _apply_pivot(T, basis, pivrow, pivcol, tol=1e-12):
 
     Parameters
     ----------
-    T : 2D array
-        A 2D array representing the simplex tableau, T, corresponding to the
+    T : 2-D array
+        A 2-D array representing the simplex tableau, T, corresponding to the
         linear programming problem. It should have the form:
 
         [[A[0, 0], A[0, 1], ..., A[0, n_total], b[0]],
@@ -172,7 +200,7 @@ def _apply_pivot(T, basis, pivrow, pivcol, tol=1e-12):
          for a Phase 1 problem (a problem in which a basic feasible solution is
          sought prior to maximizing the actual objective. ``T`` is modified in
          place by ``_solve_simplex``.
-    basis : 1D array
+    basis : 1-D array
         An array of the indices of the basic variables, such that basis[i]
         contains the column corresponding to the basic variable for row i.
         Basis is modified in place by _apply_pivot.
@@ -198,11 +226,12 @@ def _apply_pivot(T, basis, pivrow, pivcol, tol=1e-12):
             "Removing redundant constraints, changing the pivot strategy "
             "via Bland's rule or increasing the tolerance may "
             "help reduce the issue.".format(pivval, tol))
-        warn(message, OptimizeWarning)
+        warn(message, OptimizeWarning, stacklevel=5)
 
 
-def _solve_simplex(T, n, basis, maxiter=1000, phase=2, status=0, message='',
-                   callback=None, tol=1.0E-12, nit0=0, bland=False, _T_o=None):
+def _solve_simplex(T, n, basis, callback, postsolve_args,
+                   maxiter=1000, tol=1e-9, phase=2, bland=False, nit0=0,
+                   ):
     """
     Solve a linear programming problem in "standard form" using the Simplex
     Method. Linear Programming is intended to solve the following problem form:
@@ -218,8 +247,8 @@ def _solve_simplex(T, n, basis, maxiter=1000, phase=2, status=0, message='',
 
     Parameters
     ----------
-    T : 2D array
-        A 2D array representing the simplex tableau, T, corresponding to the
+    T : 2-D array
+        A 2-D array representing the simplex tableau, T, corresponding to the
         linear programming problem. It should have the form:
 
         [[A[0, 0], A[0, 1], ..., A[0, n_total], b[0]],
@@ -246,34 +275,27 @@ def _solve_simplex(T, n, basis, maxiter=1000, phase=2, status=0, message='',
          place by ``_solve_simplex``.
     n : int
         The number of true variables in the problem.
-    basis : 1D array
+    basis : 1-D array
         An array of the indices of the basic variables, such that basis[i]
         contains the column corresponding to the basic variable for row i.
         Basis is modified in place by _solve_simplex
-    maxiter : int
-        The maximum number of iterations to perform before aborting the
-        optimization.
-    phase : int
-        The phase of the optimization being executed. In phase 1 a basic
-        feasible solution is sought and the T has an additional row
-        representing an alternate objective function.
-    callback : callable, optional (simplex only)
+    callback : callable, optional
         If a callback function is provided, it will be called within each
-        iteration of the simplex algorithm. The callback must require a
+        iteration of the algorithm. The callback must accept a
         `scipy.optimize.OptimizeResult` consisting of the following fields:
 
-            x : 1D array
-                The independent variable vector which optimizes the linear
-                programming problem.
+            x : 1-D array
+                Current solution vector
             fun : float
-                Value of the objective function.
+                Current value of the objective function
             success : bool
-                True if the algorithm succeeded in finding an optimal solution.
-            slack : 1D array
+                True only when a phase has completed successfully. This
+                will be False for most iterations.
+            slack : 1-D array
                 The values of the slack variables. Each slack variable
                 corresponds to an inequality constraint. If the slack is zero,
                 the corresponding constraint is active.
-            con : 1D array
+            con : 1-D array
                 The (nominally zero) residuals of the equality constraints,
                 that is, ``b - A_eq @ x``
             phase : int
@@ -293,17 +315,27 @@ def _solve_simplex(T, n, basis, maxiter=1000, phase=2, status=0, message='',
                 The number of iterations performed.
             message : str
                 A string descriptor of the exit status of the optimization.
+    postsolve_args : tuple
+        Data needed by _postsolve to convert the solution to the standard-form
+        problem into the solution to the original problem.
+    maxiter : int
+        The maximum number of iterations to perform before aborting the
+        optimization.
     tol : float
         The tolerance which determines when a solution is "close enough" to
         zero in Phase 1 to be considered a basic feasible solution or close
         enough to positive to serve as an optimal solution.
-    nit0 : int
-        The initial iteration number used to keep an accurate iteration total
-        in a two-phase problem.
+    phase : int
+        The phase of the optimization being executed. In phase 1 a basic
+        feasible solution is sought and the T has an additional row
+        representing an alternate objective function.
     bland : bool
         If True, choose pivots using Bland's rule [3]_. In problems which
         fail to converge due to cycling, using Bland's rule can provide
         convergence at the expense of a less optimal path about the simplex.
+    nit0 : int
+        The initial iteration number used to keep an accurate iteration total
+        in a two-phase problem.
 
     Returns
     -------
@@ -321,12 +353,14 @@ def _solve_simplex(T, n, basis, maxiter=1000, phase=2, status=0, message='',
 
     """
     nit = nit0
+    status = 0
+    message = ''
     complete = False
 
     if phase == 1:
-        m = T.shape[0]-2
+        m = T.shape[1]-2
     elif phase == 2:
-        m = T.shape[0]-1
+        m = T.shape[1]-1
     else:
         raise ValueError("Argument 'phase' to _solve_simplex must be 1 or 2")
 
@@ -345,7 +379,7 @@ def _solve_simplex(T, n, basis, maxiter=1000, phase=2, status=0, message='',
                             if abs(T[pivrow, col]) > tol]
             if len(non_zero_row) > 0:
                 pivcol = non_zero_row[0]
-                _apply_pivot(T, basis, pivrow, pivcol)
+                _apply_pivot(T, basis, pivrow, pivcol, tol)
                 nit += 1
 
     if len(basis[:m]) == 0:
@@ -370,11 +404,11 @@ def _solve_simplex(T, n, basis, maxiter=1000, phase=2, status=0, message='',
                 complete = True
 
         if callback is not None:
+            solution[:] = 0
             solution[basis[:n]] = T[:n, -1]
             x = solution[:m]
-            c, A_ub, b_ub, A_eq, b_eq, bounds, undo = _T_o
-            x, fun, slack, con, _, _ = _postsolve(
-                x, c, A_ub, b_ub, A_eq, b_eq, bounds, undo=undo, tol=tol
+            x, fun, slack, con, _ = _postsolve(
+                x, postsolve_args, tol=tol
             )
             res = OptimizeResult({
                 'x': x,
@@ -396,13 +430,14 @@ def _solve_simplex(T, n, basis, maxiter=1000, phase=2, status=0, message='',
                 status = 1
                 complete = True
             else:
-                _apply_pivot(T, basis, pivrow, pivcol)
+                _apply_pivot(T, basis, pivrow, pivcol, tol)
                 nit += 1
     return nit, status
 
 
-def _linprog_simplex(c, c0, A, b, maxiter=1000, disp=False, callback=None,
-                     tol=1.0E-12, bland=False, _T_o=None, **unknown_options):
+def _linprog_simplex(c, c0, A, b, callback, postsolve_args,
+                     maxiter=1000, tol=1e-9, disp=False, bland=False,
+                     **unknown_options):
     """
     Minimize a linear objective function subject to linear equality and
     non-negativity constraints using the two phase simplex method.
@@ -419,53 +454,53 @@ def _linprog_simplex(c, c0, A, b, maxiter=1000, disp=False, callback=None,
 
     Parameters
     ----------
-    c : 1D array
+    c : 1-D array
         Coefficients of the linear objective function to be minimized.
     c0 : float
         Constant term in objective function due to fixed (and eliminated)
         variables. (Purely for display.)
-    A : 2D array
-        2D array such that ``A @ x``, gives the values of the equality
+    A : 2-D array
+        2-D array such that ``A @ x``, gives the values of the equality
         constraints at ``x``.
-    b : 1D array
-        1D array of values representing the right hand side of each equality
+    b : 1-D array
+        1-D array of values representing the right hand side of each equality
         constraint (row) in ``A``.
-    callback : callable, optional (simplex only)
+    callback : callable, optional
         If a callback function is provided, it will be called within each
-        iteration of the simplex algorithm. The callback must require a
+        iteration of the algorithm. The callback function must accept a single
         `scipy.optimize.OptimizeResult` consisting of the following fields:
 
-            x : 1D array
-                The independent variable vector which optimizes the linear
-                programming problem.
+            x : 1-D array
+                Current solution vector
             fun : float
-                Value of the objective function.
+                Current value of the objective function
             success : bool
-                True if the algorithm succeeded in finding an optimal solution.
-            slack : 1D array
+                True when an algorithm has completed successfully.
+            slack : 1-D array
                 The values of the slack variables. Each slack variable
                 corresponds to an inequality constraint. If the slack is zero,
                 the corresponding constraint is active.
-            con : 1D array
-                The (nominally zero) residuals of the equality constraints, that
-                is, ``b - A_eq @ x``
+            con : 1-D array
+                The (nominally zero) residuals of the equality constraints,
+                that is, ``b - A_eq @ x``
             phase : int
-                The phase of the optimization being executed. In phase 1 a basic
-                feasible solution is sought and the T has an additional row
-                representing an alternate objective function.
+                The phase of the algorithm being executed.
             status : int
-                An integer representing the exit status of the optimization::
+                An integer representing the status of the optimization::
 
-                     0 : Optimization terminated successfully
+                     0 : Algorithm proceeding nominally
                      1 : Iteration limit reached
                      2 : Problem appears to be infeasible
                      3 : Problem appears to be unbounded
                      4 : Serious numerical difficulties encountered
-
             nit : int
                 The number of iterations performed.
             message : str
                 A string descriptor of the exit status of the optimization.
+    postsolve_args : tuple
+        Data needed by _postsolve to convert the solution to the standard-form
+        problem into the solution to the original problem.
+
     Options
     -------
     maxiter : int
@@ -481,10 +516,14 @@ def _linprog_simplex(c, c0, A, b, maxiter=1000, disp=False, callback=None,
         prevent cycling. If False, choose pivots which should lead to a
         converged solution more quickly. The latter method is subject to
         cycling (non-convergence) in rare instances.
+    unknown_options : dict
+        Optional arguments not used by this particular solver. If
+        `unknown_options` is non-empty a warning is issued listing all
+        unused options.
 
     Returns
     -------
-    x : 1D array
+    x : 1-D array
         Solution vector.
     status : int
         An integer representing the exit status of the optimization::
@@ -579,10 +618,14 @@ def _linprog_simplex(c, c0, A, b, maxiter=1000, disp=False, callback=None,
     row_pseudo_objective[av] = 0
     T = np.vstack((row_constraints, row_objective, row_pseudo_objective))
 
-    nit1, status = _solve_simplex(T, n, basis, phase=1, callback=callback,
-                                  maxiter=maxiter, tol=tol, bland=bland, _T_o=_T_o)
+    nit1, status = _solve_simplex(T, n, basis, callback=callback,
+                                  postsolve_args=postsolve_args,
+                                  maxiter=maxiter, tol=tol, phase=1,
+                                  bland=bland
+                                  )
     # if pseudo objective is zero, remove the last row from the tableau and
     # proceed to phase 2
+    nit2 = nit1
     if abs(T[-1, -1]) < tol:
         # Remove the pseudo-objective row from the tableau
         T = T[:-1, :]
@@ -591,7 +634,6 @@ def _linprog_simplex(c, c0, A, b, maxiter=1000, disp=False, callback=None,
     else:
         # Failure to find a feasible starting point
         status = 2
-        nit2 = nit1
         messages[status] = (
             "Phase 1 of the simplex method failed to find a feasible "
             "solution. The pseudo-objective function evaluates to {0:.1e} "
@@ -604,13 +646,14 @@ def _linprog_simplex(c, c0, A, b, maxiter=1000, disp=False, callback=None,
 
     if status == 0:
         # Phase 2
-        nit2, status = _solve_simplex(T, n, basis, maxiter=maxiter,
-                                      phase=2, callback=callback, tol=tol,
-                                      nit0=nit1, bland=bland, _T_o=_T_o)
+        nit2, status = _solve_simplex(T, n, basis, callback=callback,
+                                      postsolve_args=postsolve_args,
+                                      maxiter=maxiter, tol=tol, phase=2,
+                                      bland=bland, nit0=nit1
+                                      )
 
     solution = np.zeros(n + m)
     solution[basis[:n]] = T[:n, -1]
     x = solution[:m]
 
     return x, status, messages[status], int(nit2)
-
