@@ -1,17 +1,16 @@
 r"""
 
-=================
 Nonlinear solvers
-=================
+-----------------
 
 .. currentmodule:: scipy.optimize
 
 This is a collection of general-purpose nonlinear multidimensional
-solvers.  These solvers find *x* for which *F(x) = 0*. Both *x*
+solvers. These solvers find *x* for which *F(x) = 0*. Both *x*
 and *F* can be multidimensional.
 
 Routines
-========
+~~~~~~~~
 
 Large-scale nonlinear solvers:
 
@@ -37,10 +36,9 @@ Simple iterations:
 
 
 Examples
-========
+~~~~~~~~
 
-Small problem
--------------
+**Small problem**
 
 >>> def F(x):
 ...    return np.cos(x) + x[::-1] - [1, 2, 3, 4]
@@ -52,8 +50,7 @@ array([ 4.04674914,  3.91158389,  2.71791677,  1.61756251])
 array([ 1.,  2.,  3.,  4.])
 
 
-Large problem
--------------
+**Large problem**
 
 Suppose that we needed to solve the following integrodifferential
 equation on the square :math:`[0,1]\times[0,1]`:
@@ -102,26 +99,23 @@ The solution can be found using the `newton_krylov` solver:
    # visualize
    import matplotlib.pyplot as plt
    x, y = mgrid[0:1:(nx*1j), 0:1:(ny*1j)]
-   plt.pcolor(x, y, sol)
+   plt.pcolormesh(x, y, sol, shading='gouraud')
    plt.colorbar()
    plt.show()
 
 """
 # Copyright (C) 2009, Pauli Virtanen <pav@iki.fi>
-# Distributed under the same license as Scipy.
-
-from __future__ import division, print_function, absolute_import
+# Distributed under the same license as SciPy.
 
 import sys
 import numpy as np
-from scipy._lib.six import callable, exec_, xrange
 from scipy.linalg import norm, solve, inv, qr, svd, LinAlgError
 from numpy import asarray, dot, vdot
 import scipy.sparse.linalg
 import scipy.sparse
 from scipy.linalg import get_blas_funcs
 import inspect
-from scipy._lib._util import getargspec_no_self as _getargspec
+from scipy._lib._util import getfullargspec_no_self as _getfullargspec
 from .linesearch import scalar_search_wolfe1, scalar_search_armijo
 
 
@@ -166,12 +160,13 @@ def _safe_norm(v):
 # Generic nonlinear solver machinery
 #------------------------------------------------------------------------------
 
+
 _doc_parts = dict(
     params_basic="""
     F : function(x) -> f
         Function whose root to find; should take and return an array-like
         object.
-    x0 : array_like
+    xin : array_like
         Initial guess for the solution
     """.strip(),
     params_extra="""
@@ -263,10 +258,12 @@ def nonlin_solve(F, x0, jacobian='krylov', iter=None, verbose=False,
     ----------
     .. [KIM] C. T. Kelley, \"Iterative Methods for Linear and Nonlinear
        Equations\". Society for Industrial and Applied Mathematics. (1995)
-       http://www.siam.org/books/kelley/
+       https://archive.siam.org/books/kelley/fr16/
 
     """
-
+    # Can't use default parameters because it's being explicitly passed as None
+    # from the calling function, so we need to set it here.
+    tol_norm = maxnorm if tol_norm is None else tol_norm
     condition = TerminationCondition(f_tol=f_tol, f_rtol=f_rtol,
                                      x_tol=x_tol, x_rtol=x_rtol,
                                      iter=iter, norm=tol_norm)
@@ -302,7 +299,7 @@ def nonlin_solve(F, x0, jacobian='krylov', iter=None, verbose=False,
     eta_treshold = 0.1
     eta = 1e-3
 
-    for n in xrange(maxiter):
+    for n in range(maxiter):
         status = condition.check(Fx, x, dx)
         if status:
             break
@@ -342,8 +339,8 @@ def nonlin_solve(F, x0, jacobian='krylov', iter=None, verbose=False,
 
         # Print status
         if verbose:
-            sys.stdout.write("%d:  |F(x)| = %g; step %g; tol %g\n" % (
-                n, norm(Fx), s, eta))
+            sys.stdout.write("%d:  |F(x)| = %g; step %g\n" % (
+                n, tol_norm(Fx), s))
             sys.stdout.flush()
     else:
         if raise_exception:
@@ -366,6 +363,7 @@ def nonlin_solve(F, x0, jacobian='krylov', iter=None, verbose=False,
     else:
         return _array_like(x, x0)
 
+
 _set_doc(nonlin_solve)
 
 
@@ -444,10 +442,7 @@ class TerminationCondition(object):
         self.f_tol = f_tol
         self.f_rtol = f_rtol
 
-        if norm is None:
-            self.norm = maxnorm
-        else:
-            self.norm = norm
+        self.norm = norm
 
         self.iter = iter
 
@@ -467,7 +462,7 @@ class TerminationCondition(object):
             return 1
 
         if self.iter is not None:
-            # backwards compatibility with Scipy 0.6.0
+            # backwards compatibility with SciPy 0.6.0
             return 2 * (self.iteration > self.iter)
 
         # NB: condition must succeed for rtol=inf even if norm == 0
@@ -486,7 +481,7 @@ class Jacobian(object):
     Common interface for Jacobians or Jacobian approximations.
 
     The optional methods come useful when implementing trust region
-    etc.  algorithms that often require evaluating transposes of the
+    etc., algorithms that often require evaluating transposes of the
     Jacobian.
 
     Methods
@@ -546,7 +541,7 @@ class Jacobian(object):
         self.dtype = F.dtype
         if self.__class__.setup is Jacobian.setup:
             # Call on the first point unless overridden
-            self.update(self, x, F)
+            self.update(x, F)
 
 
 class InverseJacobian(object):
@@ -846,7 +841,7 @@ class LowRankMatrix(object):
            systems of nonlinear equations\". Mathematisch Instituut,
            Universiteit Leiden, The Netherlands (2003).
 
-           http://www.math.leidenuniv.nl/scripties/Rotten.pdf
+           https://web.archive.org/web/20161022015821/http://www.math.leidenuniv.nl/scripties/Rotten.pdf
 
         """
         if self.collapsed is not None:
@@ -878,13 +873,14 @@ class LowRankMatrix(object):
         C = dot(C, inv(WH))
         D = dot(D, WH.T.conj())
 
-        for k in xrange(q):
+        for k in range(q):
             self.cs[k] = C[:,k].copy()
             self.ds[k] = D[:,k].copy()
 
         del self.cs[q:]
         del self.ds[q:]
 
+
 _doc_parts['broyden_params'] = """
     alpha : float, optional
         Initial guess for the Jacobian is ``(-1/alpha)``.
@@ -905,7 +901,7 @@ _doc_parts['broyden_params'] = """
 
     max_rank : int, optional
         Maximum rank for the Broyden matrix.
-        Default is infinity (ie., no rank reduction).
+        Default is infinity (i.e., no rank reduction).
     """.strip()
 
 
@@ -921,6 +917,11 @@ class BroydenFirst(GenericBroyden):
     %(broyden_params)s
     %(params_extra)s
 
+    See Also
+    --------
+    root : Interface to root finding algorithms for multivariate
+           functions. See ``method=='broyden1'`` in particular.
+
     Notes
     -----
     This algorithm implements the inverse Jacobian Quasi-Newton update
@@ -939,7 +940,7 @@ class BroydenFirst(GenericBroyden):
        systems of nonlinear equations\". Mathematisch Instituut,
        Universiteit Leiden, The Netherlands (2003).
 
-       http://www.math.leidenuniv.nl/scripties/Rotten.pdf
+       https://web.archive.org/web/20161022015821/http://www.math.leidenuniv.nl/scripties/Rotten.pdf
 
     """
 
@@ -1014,11 +1015,16 @@ class BroydenSecond(BroydenFirst):
     %(broyden_params)s
     %(params_extra)s
 
+    See Also
+    --------
+    root : Interface to root finding algorithms for multivariate
+           functions. See ``method=='broyden2'`` in particular.
+
     Notes
     -----
     This algorithm implements the inverse Jacobian Quasi-Newton update
 
-    .. math:: H_+ = H + (dx - H df) df^\dagger / ( df^\dagger df)
+    .. math:: H_+ = H + (dx - H df) df^\\dagger / ( df^\\dagger df)
 
     corresponding to Broyden's second method.
 
@@ -1029,7 +1035,7 @@ class BroydenSecond(BroydenFirst):
        systems of nonlinear equations\". Mathematisch Instituut,
        Universiteit Leiden, The Netherlands (2003).
 
-       http://www.math.leidenuniv.nl/scripties/Rotten.pdf
+       https://web.archive.org/web/20161022015821/http://www.math.leidenuniv.nl/scripties/Rotten.pdf
 
     """
 
@@ -1066,6 +1072,11 @@ class Anderson(GenericBroyden):
         Compared to unity, good values of the order of 0.01.
     %(params_extra)s
 
+    See Also
+    --------
+    root : Interface to root finding algorithms for multivariate
+           functions. See ``method=='anderson'`` in particular.
+
     References
     ----------
     .. [Ey] V. Eyert, J. Comp. Phys., 124, 271 (1996).
@@ -1080,7 +1091,7 @@ class Anderson(GenericBroyden):
     #     A      = W + dF^H dF
     #     W      = w0^2 diag(dF^H dF)
     #
-    # so that for w0 = 0 the secant condition applies for last M iterates, ie.,
+    # so that for w0 = 0 the secant condition applies for last M iterates, i.e.,
     #
     #     J^-1 df_j = dx_j
     #
@@ -1114,7 +1125,7 @@ class Anderson(GenericBroyden):
             return dx
 
         df_f = np.empty(n, dtype=f.dtype)
-        for k in xrange(n):
+        for k in range(n):
             df_f[k] = vdot(self.df[k], f)
 
         try:
@@ -1125,7 +1136,7 @@ class Anderson(GenericBroyden):
             del self.df[:]
             return dx
 
-        for m in xrange(n):
+        for m in range(n):
             dx += gamma[m]*(self.dx[m] + self.alpha*self.df[m])
         return dx
 
@@ -1137,18 +1148,18 @@ class Anderson(GenericBroyden):
             return dx
 
         df_f = np.empty(n, dtype=f.dtype)
-        for k in xrange(n):
+        for k in range(n):
             df_f[k] = vdot(self.df[k], f)
 
         b = np.empty((n, n), dtype=f.dtype)
-        for i in xrange(n):
-            for j in xrange(n):
+        for i in range(n):
+            for j in range(n):
                 b[i,j] = vdot(self.df[i], self.dx[j])
                 if i == j and self.w0 != 0:
                     b[i,j] -= vdot(self.df[i], self.df[i])*self.w0**2*self.alpha
         gamma = solve(b, df_f)
 
-        for m in xrange(n):
+        for m in range(n):
             dx += gamma[m]*(self.df[m] + self.dx[m]/self.alpha)
         return dx
 
@@ -1166,8 +1177,8 @@ class Anderson(GenericBroyden):
         n = len(self.dx)
         a = np.zeros((n, n), dtype=f.dtype)
 
-        for i in xrange(n):
-            for j in xrange(i, n):
+        for i in range(n):
+            for j in range(i, n):
                 if i == j:
                     wd = self.w0**2
                 else:
@@ -1200,6 +1211,11 @@ class DiagBroyden(GenericBroyden):
     alpha : float, optional
         Initial guess for the Jacobian is (-1/alpha).
     %(params_extra)s
+
+    See Also
+    --------
+    root : Interface to root finding algorithms for multivariate
+           functions. See ``method=='diagbroyden'`` in particular.
     """
 
     def __init__(self, alpha=None):
@@ -1208,7 +1224,7 @@ class DiagBroyden(GenericBroyden):
 
     def setup(self, x, F, func):
         GenericBroyden.setup(self, x, F, func)
-        self.d = np.ones((self.shape[0],), dtype=self.dtype) / self.alpha
+        self.d = np.full((self.shape[0],), 1 / self.alpha, dtype=self.dtype)
 
     def solve(self, f, tol=0):
         return -f / self.d
@@ -1244,6 +1260,12 @@ class LinearMixing(GenericBroyden):
     alpha : float, optional
         The Jacobian approximation is (-1/alpha).
     %(params_extra)s
+
+    See Also
+    --------
+    root : Interface to root finding algorithms for multivariate
+           functions. See ``method=='linearmixing'`` in particular.
+
     """
 
     def __init__(self, alpha=None):
@@ -1263,7 +1285,7 @@ class LinearMixing(GenericBroyden):
         return -f/np.conj(self.alpha)
 
     def todense(self):
-        return np.diag(-np.ones(self.shape[0])/self.alpha)
+        return np.diag(np.full(self.shape[0], -1/self.alpha))
 
     def _update(self, x, f, dx, df, dx_norm, df_norm):
         pass
@@ -1280,6 +1302,11 @@ class ExcitingMixing(GenericBroyden):
        This algorithm may be useful for specific problems, but whether
        it will work may depend strongly on the problem.
 
+    See Also
+    --------
+    root : Interface to root finding algorithms for multivariate
+           functions. See ``method=='excitingmixing'`` in particular.
+
     Parameters
     ----------
     %(params_basic)s
@@ -1299,7 +1326,7 @@ class ExcitingMixing(GenericBroyden):
 
     def setup(self, x, F, func):
         GenericBroyden.setup(self, x, F, func)
-        self.beta = self.alpha * np.ones((self.shape[0],), dtype=self.dtype)
+        self.beta = np.full((self.shape[0],), self.alpha, dtype=self.dtype)
 
     def solve(self, f, tol=0):
         return -f*self.beta
@@ -1344,6 +1371,10 @@ class KrylovJacobian(Jacobian):
         the iterative solvers in `scipy.sparse.linalg`.
 
         The default is `scipy.sparse.linalg.lgmres`.
+    inner_maxiter : int, optional
+        Parameter to pass to the "inner" Krylov solver: maximum number of
+        iterations. Iteration will stop after maxiter steps even if the
+        specified tolerance has not been achieved.
     inner_M : LinearOperator or InverseJacobian
         Preconditioner for the inner Krylov iteration.
         Note that you can use also inverse Jacobians as (adaptive)
@@ -1357,16 +1388,20 @@ class KrylovJacobian(Jacobian):
         If the preconditioner has a method named 'update', it will be called
         as ``update(x, f)`` after each nonlinear step, with ``x`` giving
         the current point, and ``f`` the current function value.
-    inner_tol, inner_maxiter, ...
-        Parameters to pass on to the \"inner\" Krylov solver.
-        See `scipy.sparse.linalg.gmres` for details.
     outer_k : int, optional
         Size of the subspace kept across LGMRES nonlinear iterations.
         See `scipy.sparse.linalg.lgmres` for details.
+    inner_kwargs : kwargs
+        Keyword parameters for the "inner" Krylov solver
+        (defined with `method`). Parameter names must start with
+        the `inner_` prefix which will be stripped before passing on
+        the inner method. See, e.g., `scipy.sparse.linalg.gmres` for details.
     %(params_extra)s
 
     See Also
     --------
+    root : Interface to root finding algorithms for multivariate
+           functions. See ``method=='krylov'`` in particular.
     scipy.sparse.linalg.gmres
     scipy.sparse.linalg.lgmres
 
@@ -1382,7 +1417,7 @@ class KrylovJacobian(Jacobian):
     Due to the use of iterative matrix inverses, these methods can
     deal with large nonlinear problems.
 
-    Scipy's `scipy.sparse.linalg` module offers a selection of Krylov
+    SciPy's `scipy.sparse.linalg` module offers a selection of Krylov
     solvers to choose from. The default here is `lgmres`, which is a
     variant of restarted GMRES iteration that reuses some of the
     information obtained in the previous Newton steps to invert
@@ -1393,9 +1428,11 @@ class KrylovJacobian(Jacobian):
 
     References
     ----------
-    .. [1] D.A. Knoll and D.E. Keyes, J. Comp. Phys. 193, 357 (2003).
+    .. [1] D.A. Knoll and D.E. Keyes, J. Comp. Phys. 193, 357 (2004).
+           :doi:`10.1016/j.jcp.2003.08.010`
     .. [2] A.H. Baker and E.R. Jessup and T. Manteuffel,
-             SIAM J. Matrix Anal. Appl. 26, 962 (2005).
+           SIAM J. Matrix Anal. Appl. 26, 962 (2005).
+           :doi:`10.1137/S0895479803422014`
 
     """
 
@@ -1417,19 +1454,24 @@ class KrylovJacobian(Jacobian):
             # Replace GMRES's outer iteration with Newton steps
             self.method_kw['restrt'] = inner_maxiter
             self.method_kw['maxiter'] = 1
+            self.method_kw.setdefault('atol', 0)
+        elif self.method is scipy.sparse.linalg.gcrotmk:
+            self.method_kw.setdefault('atol', 0)
         elif self.method is scipy.sparse.linalg.lgmres:
             self.method_kw['outer_k'] = outer_k
             # Replace LGMRES's outer iteration with Newton steps
             self.method_kw['maxiter'] = 1
             # Carry LGMRES's `outer_v` vectors across nonlinear iterations
             self.method_kw.setdefault('outer_v', [])
+            self.method_kw.setdefault('prepend_outer_v', True)
             # But don't carry the corresponding Jacobian*v products, in case
             # the Jacobian changes a lot in the nonlinear step
             #
             # XXX: some trust-region inspired ideas might be more efficient...
-            #      See eg. Brown & Saad. But needs to be implemented separately
+            #      See e.g., Brown & Saad. But needs to be implemented separately
             #      since it's not an inexact Newton method.
             self.method_kw.setdefault('store_outer_Av', False)
+            self.method_kw.setdefault('atol', 0)
 
         for key, value in kw.items():
             if not key.startswith('inner_'):
@@ -1491,14 +1533,15 @@ class KrylovJacobian(Jacobian):
 
 def _nonlin_wrapper(name, jac):
     """
-    Construct a solver wrapper with given name and jacobian approx.
+    Construct a solver wrapper with given name and Jacobian approx.
 
     It inspects the keyword arguments of ``jac.__init__``, and allows to
     use the same arguments in the wrapper function, in addition to the
     keyword arguments of `nonlin_solve`
 
     """
-    args, varargs, varkw, defaults = _getargspec(jac.__init__)
+    signature = _getfullargspec(jac.__init__)
+    args, varargs, varkw, defaults, kwonlyargs, kwdefaults, _ = signature
     kwargs = list(zip(args[-len(defaults):], defaults))
     kw_str = ", ".join(["%s=%r" % (k, v) for k, v in kwargs])
     if kw_str:
@@ -1506,6 +1549,8 @@ def _nonlin_wrapper(name, jac):
     kwkw_str = ", ".join(["%s=%s" % (k, k) for k, v in kwargs])
     if kwkw_str:
         kwkw_str = kwkw_str + ", "
+    if kwonlyargs:
+        raise ValueError('Unexpected signature %s' % signature)
 
     # Construct the wrapper function so that its keyword arguments
     # are visible in pydoc.help etc.
@@ -1523,12 +1568,13 @@ def %(name)s(F, xin, iter=None %(kw)s, verbose=False, maxiter=None,
                              kwkw=kwkw_str)
     ns = {}
     ns.update(globals())
-    exec_(wrapper, ns)
+    exec(wrapper, ns)
     func = ns[name]
     func.__doc__ = jac.__doc__
     _set_doc(func)
     return func
 
+
 broyden1 = _nonlin_wrapper('broyden1', BroydenFirst)
 broyden2 = _nonlin_wrapper('broyden2', BroydenSecond)
 anderson = _nonlin_wrapper('anderson', Anderson)
