@@ -28,11 +28,11 @@
 # NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 # SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-from __future__ import division, print_function, absolute_import
+from collections.abc import Iterable
 import warnings
-import numbers
 import numpy
 import operator
+from numpy.core.multiarray import normalize_axis_index
 from . import _ni_support
 from . import _nd_image
 from . import _ni_docstrings
@@ -54,7 +54,7 @@ def _invalid_origin(origin, lenw):
 @_ni_docstrings.docfiller
 def correlate1d(input, weights, axis=-1, output=None, mode="reflect",
                 cval=0.0, origin=0):
-    """Calculate a one-dimensional correlation along the given axis.
+    """Calculate a 1-D correlation along the given axis.
 
     The lines of the array along the given axis are correlated with the
     given weights.
@@ -63,7 +63,7 @@ def correlate1d(input, weights, axis=-1, output=None, mode="reflect",
     ----------
     %(input)s
     weights : array
-        One-dimensional sequence of numbers.
+        1-D sequence of numbers.
     %(axis)s
     %(output)s
     %(mode)s
@@ -85,7 +85,7 @@ def correlate1d(input, weights, axis=-1, output=None, mode="reflect",
         raise RuntimeError('no filter weights given')
     if not weights.flags.contiguous:
         weights = weights.copy()
-    axis = _ni_support._check_axis(axis, input.ndim)
+    axis = normalize_axis_index(axis, input.ndim)
     if _invalid_origin(origin, len(weights)):
         raise ValueError('Invalid origin; origin must satisfy '
                          '-(len(weights) // 2) <= origin <= '
@@ -99,7 +99,7 @@ def correlate1d(input, weights, axis=-1, output=None, mode="reflect",
 @_ni_docstrings.docfiller
 def convolve1d(input, weights, axis=-1, output=None, mode="reflect",
                cval=0.0, origin=0):
-    """Calculate a one-dimensional convolution along the given axis.
+    """Calculate a 1-D convolution along the given axis.
 
     The lines of the array along the given axis are convolved with the
     given weights.
@@ -108,7 +108,7 @@ def convolve1d(input, weights, axis=-1, output=None, mode="reflect",
     ----------
     %(input)s
     weights : ndarray
-        One-dimensional sequence of numbers.
+        1-D sequence of numbers.
     %(axis)s
     %(output)s
     %(mode)s
@@ -135,29 +135,39 @@ def convolve1d(input, weights, axis=-1, output=None, mode="reflect",
 
 def _gaussian_kernel1d(sigma, order, radius):
     """
-    Computes a 1D Gaussian convolution kernel.
+    Computes a 1-D Gaussian convolution kernel.
     """
     if order < 0:
         raise ValueError('order must be non-negative')
-    p = numpy.polynomial.Polynomial([0, 0, -0.5 / (sigma * sigma)])
-    x = numpy.arange(-radius, radius + 1)
-    phi_x = numpy.exp(p(x), dtype=numpy.double)
-    phi_x /= phi_x.sum()
-    if order > 0:
-        q = numpy.polynomial.Polynomial([1])
-        p_deriv = p.deriv()
+    exponent_range = numpy.arange(order + 1)
+    sigma2 = sigma * sigma
+    x = numpy.arange(-radius, radius+1)
+    phi_x = numpy.exp(-0.5 / sigma2 * x ** 2)
+    phi_x = phi_x / phi_x.sum()
+
+    if order == 0:
+        return phi_x
+    else:
+        # f(x) = q(x) * phi(x) = q(x) * exp(p(x))
+        # f'(x) = (q'(x) + q(x) * p'(x)) * phi(x)
+        # p'(x) = -1 / sigma ** 2
+        # Implement q'(x) + q(x) * p'(x) as a matrix operator and apply to the
+        # coefficients of q(x)
+        q = numpy.zeros(order + 1)
+        q[0] = 1
+        D = numpy.diag(exponent_range[1:], 1)  # D @ q(x) = q'(x)
+        P = numpy.diag(numpy.ones(order)/-sigma2, -1)  # P @ q(x) = q(x) * p'(x)
+        Q_deriv = D + P
         for _ in range(order):
-            # f(x) = q(x) * phi(x) = q(x) * exp(p(x))
-            # f'(x) = (q'(x) + q(x) * p'(x)) * phi(x)
-            q = q.deriv() + q * p_deriv
-        phi_x *= q(x)
-    return phi_x
+            q = Q_deriv.dot(q)
+        q = (x[:, None] ** exponent_range).dot(q)
+        return q * phi_x
 
 
 @_ni_docstrings.docfiller
 def gaussian_filter1d(input, sigma, axis=-1, order=0, output=None,
                       mode="reflect", cval=0.0, truncate=4.0):
-    """One-dimensional Gaussian filter.
+    """1-D Gaussian filter.
 
     Parameters
     ----------
@@ -222,7 +232,7 @@ def gaussian_filter(input, sigma, order=0, output=None,
         all axes.
     order : int or sequence of ints, optional
         The order of the filter along each axis is given as a sequence
-        of integers, or as a single number.  An order of 0 corresponds
+        of integers, or as a single number. An order of 0 corresponds
         to convolution with a Gaussian kernel. A positive order
         corresponds to convolution with that derivative of a Gaussian.
     %(output)s
@@ -240,7 +250,7 @@ def gaussian_filter(input, sigma, order=0, output=None,
     Notes
     -----
     The multidimensional filter is implemented as a sequence of
-    one-dimensional convolution filters. The intermediate arrays are
+    1-D convolution filters. The intermediate arrays are
     stored in the same data type as the output. Therefore, for output
     types with a limited precision, the results may be imprecise
     because intermediate results may be stored with insufficient
@@ -320,7 +330,7 @@ def prewitt(input, axis=-1, output=None, mode="reflect", cval=0.0):
     >>> plt.show()
     """
     input = numpy.asarray(input)
-    axis = _ni_support._check_axis(axis, input.ndim)
+    axis = normalize_axis_index(axis, input.ndim)
     output = _ni_support._get_output(output, input)
     modes = _ni_support._normalize_sequence(mode, input.ndim)
     correlate1d(input, [-1, 0, 1], axis, output, modes[axis], cval, 0)
@@ -357,7 +367,7 @@ def sobel(input, axis=-1, output=None, mode="reflect", cval=0.0):
     >>> plt.show()
     """
     input = numpy.asarray(input)
-    axis = _ni_support._check_axis(axis, input.ndim)
+    axis = normalize_axis_index(axis, input.ndim)
     output = _ni_support._get_output(output, input)
     modes = _ni_support._normalize_sequence(mode, input.ndim)
     correlate1d(input, [-1, 0, 1], axis, output, modes[axis], cval, 0)
@@ -373,7 +383,7 @@ def generic_laplace(input, derivative2, output=None, mode="reflect",
                     extra_arguments=(),
                     extra_keywords=None):
     """
-    N-dimensional Laplace filter using a provided second derivative function.
+    N-D Laplace filter using a provided second derivative function.
 
     Parameters
     ----------
@@ -411,7 +421,7 @@ def generic_laplace(input, derivative2, output=None, mode="reflect",
 
 @_ni_docstrings.docfiller
 def laplace(input, output=None, mode="reflect", cval=0.0):
-    """N-dimensional Laplace filter based on approximate second derivatives.
+    """N-D Laplace filter based on approximate second derivatives.
 
     Parameters
     ----------
@@ -442,7 +452,7 @@ def laplace(input, output=None, mode="reflect", cval=0.0):
 @_ni_docstrings.docfiller
 def gaussian_laplace(input, sigma, output=None, mode="reflect",
                      cval=0.0, **kwargs):
-    """Multidimensional Laplace filter using gaussian second derivatives.
+    """Multidimensional Laplace filter using Gaussian second derivatives.
 
     Parameters
     ----------
@@ -545,7 +555,7 @@ def gaussian_gradient_magnitude(input, sigma, output=None,
     sigma : scalar or sequence of scalars
         The standard deviations of the Gaussian filter are given for
         each axis as a sequence, or as a single number, in which case
-        it is equal for all axes..
+        it is equal for all axes.
     %(output)s
     %(mode_multiple)s
     %(cval)s
@@ -608,8 +618,18 @@ def _correlate_or_convolve(input, weights, output, mode, cval, origin,
     if not weights.flags.contiguous:
         weights = weights.copy()
     output = _ni_support._get_output(output, input)
+    temp_needed = numpy.may_share_memory(input, output)
+    if temp_needed:
+        # input and output arrays cannot share memory
+        temp = output
+        output = _ni_support._get_output(output.dtype, input)
+    if not isinstance(mode, str) and isinstance(mode, Iterable):
+        raise RuntimeError("A sequence of modes is not supported")
     mode = _ni_support._extend_mode_to_code(mode)
     _nd_image.correlate(input, weights, output, mode, cval, origins)
+    if temp_needed:
+        temp[...] = output
+        output = temp
     return output
 
 
@@ -617,7 +637,7 @@ def _correlate_or_convolve(input, weights, output, mode, cval, origin,
 def correlate(input, weights, output=None, mode='reflect', cval=0.0,
               origin=0):
     """
-    Multi-dimensional correlation.
+    Multidimensional correlation.
 
     The array is correlated with the given kernel.
 
@@ -627,13 +647,50 @@ def correlate(input, weights, output=None, mode='reflect', cval=0.0,
     weights : ndarray
         array of weights, same number of dimensions as input
     %(output)s
-    %(mode_multiple)s
+    %(mode)s
     %(cval)s
     %(origin_multiple)s
 
+    Returns
+    -------
+    result : ndarray
+        The result of correlation of `input` with `weights`.
+
     See Also
     --------
     convolve : Convolve an image with a kernel.
+
+    Examples
+    --------
+    Correlation is the process of moving a filter mask often referred to
+    as kernel over the image and computing the sum of products at each location.
+
+    >>> from scipy.ndimage import correlate
+    >>> input_img = np.arange(25).reshape(5,5)
+    >>> print(input_img)
+    [[ 0  1  2  3  4]
+    [ 5  6  7  8  9]
+    [10 11 12 13 14]
+    [15 16 17 18 19]
+    [20 21 22 23 24]]
+
+    Define a kernel (weights) for correlation. In this example, it is for sum of
+    center and up, down, left and right next elements.
+
+    >>> weights = [[0, 1, 0],
+    ...            [1, 1, 1],
+    ...            [0, 1, 0]]
+
+    We can calculate a correlation result:
+    For example, element ``[2,2]`` is ``7 + 11 + 12 + 13 + 17 = 60``.
+
+    >>> correlate(input_img, weights)
+    array([[  6,  10,  15,  20,  24],
+        [ 26,  30,  35,  40,  44],
+        [ 51,  55,  60,  65,  69],
+        [ 76,  80,  85,  90,  94],
+        [ 96, 100, 105, 110, 114]])
+
     """
     return _correlate_or_convolve(input, weights, output, mode, cval,
                                   origin, False)
@@ -653,7 +710,7 @@ def convolve(input, weights, output=None, mode='reflect', cval=0.0,
     weights : array_like
         Array of weights, same number of dimensions as input
     %(output)s
-    %(mode_multiple)s
+    %(mode)s
     cval : scalar, optional
         Value to fill past edges of input if `mode` is 'constant'. Default
         is 0.0
@@ -672,14 +729,14 @@ def convolve(input, weights, output=None, mode='reflect', cval=0.0,
     -----
     Each value in result is :math:`C_i = \\sum_j{I_{i+k-j} W_j}`, where
     W is the `weights` kernel,
-    j is the n-D spatial index over :math:`W`,
+    j is the N-D spatial index over :math:`W`,
     I is the `input` and k is the coordinate of the center of
     W, specified by `origin` in the input parameters.
 
     Examples
     --------
     Perhaps the simplest case to understand is ``mode='constant', cval=0.0``,
-    because in this case borders (i.e. where the `weights` kernel, centered
+    because in this case borders (i.e., where the `weights` kernel, centered
     on any one value, extends beyond an edge of `input`) are treated as zeros.
 
     >>> a = np.array([[1, 2, 0, 0],
@@ -748,7 +805,7 @@ def convolve(input, weights, output=None, mode='reflect', cval=0.0,
 @_ni_docstrings.docfiller
 def uniform_filter1d(input, size, axis=-1, output=None,
                      mode="reflect", cval=0.0, origin=0):
-    """Calculate a one-dimensional uniform filter along the given axis.
+    """Calculate a 1-D uniform filter along the given axis.
 
     The lines of the array along the given axis are filtered with a
     uniform filter of given size.
@@ -773,7 +830,7 @@ def uniform_filter1d(input, size, axis=-1, output=None,
     input = numpy.asarray(input)
     if numpy.iscomplexobj(input):
         raise TypeError('Complex type not supported')
-    axis = _ni_support._check_axis(axis, input.ndim)
+    axis = normalize_axis_index(axis, input.ndim)
     if size < 1:
         raise RuntimeError('incorrect filter size')
     output = _ni_support._get_output(output, input)
@@ -788,7 +845,7 @@ def uniform_filter1d(input, size, axis=-1, output=None,
 @_ni_docstrings.docfiller
 def uniform_filter(input, size=3, output=None, mode="reflect",
                    cval=0.0, origin=0):
-    """Multi-dimensional uniform filter.
+    """Multidimensional uniform filter.
 
     Parameters
     ----------
@@ -809,8 +866,8 @@ def uniform_filter(input, size=3, output=None, mode="reflect",
 
     Notes
     -----
-    The multi-dimensional filter is implemented as a sequence of
-    one-dimensional uniform filters. The intermediate arrays are stored
+    The multidimensional filter is implemented as a sequence of
+    1-D uniform filters. The intermediate arrays are stored
     in the same data type as the output. Therefore, for output types
     with a limited precision, the results may be imprecise because
     intermediate results may be stored with insufficient precision.
@@ -850,7 +907,7 @@ def uniform_filter(input, size=3, output=None, mode="reflect",
 @_ni_docstrings.docfiller
 def minimum_filter1d(input, size, axis=-1, output=None,
                      mode="reflect", cval=0.0, origin=0):
-    """Calculate a one-dimensional minimum filter along the given axis.
+    """Calculate a 1-D minimum filter along the given axis.
 
     The lines of the array along the given axis are filtered with a
     minimum filter of given size.
@@ -887,7 +944,7 @@ def minimum_filter1d(input, size, axis=-1, output=None,
     input = numpy.asarray(input)
     if numpy.iscomplexobj(input):
         raise TypeError('Complex type not supported')
-    axis = _ni_support._check_axis(axis, input.ndim)
+    axis = normalize_axis_index(axis, input.ndim)
     if size < 1:
         raise RuntimeError('incorrect filter size')
     output = _ni_support._get_output(output, input)
@@ -902,7 +959,7 @@ def minimum_filter1d(input, size, axis=-1, output=None,
 @_ni_docstrings.docfiller
 def maximum_filter1d(input, size, axis=-1, output=None,
                      mode="reflect", cval=0.0, origin=0):
-    """Calculate a one-dimensional maximum filter along the given axis.
+    """Calculate a 1-D maximum filter along the given axis.
 
     The lines of the array along the given axis are filtered with a
     maximum filter of given size.
@@ -944,7 +1001,7 @@ def maximum_filter1d(input, size, axis=-1, output=None,
     input = numpy.asarray(input)
     if numpy.iscomplexobj(input):
         raise TypeError('Complex type not supported')
-    axis = _ni_support._check_axis(axis, input.ndim)
+    axis = normalize_axis_index(axis, input.ndim)
     if size < 1:
         raise RuntimeError('incorrect filter size')
     output = _ni_support._get_output(output, input)
@@ -986,6 +1043,11 @@ def _min_or_max_filter(input, size, footprint, structure, output, mode,
     if numpy.iscomplexobj(input):
         raise TypeError('Complex type not supported')
     output = _ni_support._get_output(output, input)
+    temp_needed = numpy.may_share_memory(input, output)
+    if temp_needed:
+        # input and output arrays cannot share memory
+        temp = output
+        output = _ni_support._get_output(output.dtype, input)
     origins = _ni_support._normalize_sequence(origin, input.ndim)
     if separable:
         sizes = _ni_support._normalize_sequence(size, input.ndim)
@@ -1017,16 +1079,23 @@ def _min_or_max_filter(input, size, footprint, structure, output, mode,
                 raise RuntimeError('structure array has incorrect shape')
             if not structure.flags.contiguous:
                 structure = structure.copy()
+        if not isinstance(mode, str) and isinstance(mode, Iterable):
+            raise RuntimeError(
+                "A sequence of modes is not supported for non-separable "
+                "footprints")
         mode = _ni_support._extend_mode_to_code(mode)
         _nd_image.min_or_max_filter(input, footprint, structure, output,
                                     mode, cval, origins, minimum)
+    if temp_needed:
+        temp[...] = output
+        output = temp
     return output
 
 
 @_ni_docstrings.docfiller
 def minimum_filter(input, size=None, footprint=None, output=None,
                    mode="reflect", cval=0.0, origin=0):
-    """Calculate a multi-dimensional minimum filter.
+    """Calculate a multidimensional minimum filter.
 
     Parameters
     ----------
@@ -1042,6 +1111,11 @@ def minimum_filter(input, size=None, footprint=None, output=None,
     minimum_filter : ndarray
         Filtered array. Has the same shape as `input`.
 
+    Notes
+    -----
+    A sequence of modes (one per axis) is only supported when the footprint is
+    separable. Otherwise, a single mode string must be provided.
+
     Examples
     --------
     >>> from scipy import ndimage, misc
@@ -1063,7 +1137,7 @@ def minimum_filter(input, size=None, footprint=None, output=None,
 @_ni_docstrings.docfiller
 def maximum_filter(input, size=None, footprint=None, output=None,
                    mode="reflect", cval=0.0, origin=0):
-    """Calculate a multi-dimensional maximum filter.
+    """Calculate a multidimensional maximum filter.
 
     Parameters
     ----------
@@ -1079,6 +1153,11 @@ def maximum_filter(input, size=None, footprint=None, output=None,
     maximum_filter : ndarray
         Filtered array. Has the same shape as `input`.
 
+    Notes
+    -----
+    A sequence of modes (one per axis) is only supported when the footprint is
+    separable. Otherwise, a single mode string must be provided.
+
     Examples
     --------
     >>> from scipy import ndimage, misc
@@ -1146,16 +1225,28 @@ def _rank_filter(input, rank, size=None, footprint=None, output=None,
                               origins)
     else:
         output = _ni_support._get_output(output, input)
+        temp_needed = numpy.may_share_memory(input, output)
+        if temp_needed:
+            # input and output arrays cannot share memory
+            temp = output
+            output = _ni_support._get_output(output.dtype, input)
+        if not isinstance(mode, str) and isinstance(mode, Iterable):
+            raise RuntimeError(
+                "A sequence of modes is not supported by non-separable rank "
+                "filters")
         mode = _ni_support._extend_mode_to_code(mode)
         _nd_image.rank_filter(input, rank, footprint, output, mode, cval,
                               origins)
+        if temp_needed:
+            temp[...] = output
+            output = temp
         return output
 
 
 @_ni_docstrings.docfiller
 def rank_filter(input, rank, size=None, footprint=None, output=None,
                 mode="reflect", cval=0.0, origin=0):
-    """Calculate a multi-dimensional rank filter.
+    """Calculate a multidimensional rank filter.
 
     Parameters
     ----------
@@ -1165,7 +1256,7 @@ def rank_filter(input, rank, size=None, footprint=None, output=None,
         indicates the largest element.
     %(size_foot)s
     %(output)s
-    %(mode_multiple)s
+    %(mode)s
     %(cval)s
     %(origin_multiple)s
 
@@ -1204,7 +1295,7 @@ def median_filter(input, size=None, footprint=None, output=None,
     %(input)s
     %(size_foot)s
     %(output)s
-    %(mode_multiple)s
+    %(mode)s
     %(cval)s
     %(origin_multiple)s
 
@@ -1234,7 +1325,7 @@ def median_filter(input, size=None, footprint=None, output=None,
 @_ni_docstrings.docfiller
 def percentile_filter(input, percentile, size=None, footprint=None,
                       output=None, mode="reflect", cval=0.0, origin=0):
-    """Calculate a multi-dimensional percentile filter.
+    """Calculate a multidimensional percentile filter.
 
     Parameters
     ----------
@@ -1244,7 +1335,7 @@ def percentile_filter(input, percentile, size=None, footprint=None,
         percentile = -20 equals percentile = 80
     %(size_foot)s
     %(output)s
-    %(mode_multiple)s
+    %(mode)s
     %(cval)s
     %(origin_multiple)s
 
@@ -1275,12 +1366,12 @@ def percentile_filter(input, percentile, size=None, footprint=None,
 def generic_filter1d(input, function, filter_size, axis=-1,
                      output=None, mode="reflect", cval=0.0, origin=0,
                      extra_arguments=(), extra_keywords=None):
-    """Calculate a one-dimensional filter along the given axis.
+    """Calculate a 1-D filter along the given axis.
 
     `generic_filter1d` iterates over the lines of the array, calling the
     given function at each line. The arguments of the line are the
-    input line, and the output line. The input and output lines are 1D
-    double arrays.  The input line is extended appropriately according
+    input line, and the output line. The input and output lines are 1-D
+    double arrays. The input line is extended appropriately according
     to the filter size and origin. The output line must be modified
     in-place with the result.
 
@@ -1343,7 +1434,7 @@ def generic_filter1d(input, function, filter_size, axis=-1,
     output = _ni_support._get_output(output, input)
     if filter_size < 1:
         raise RuntimeError('invalid filter size')
-    axis = _ni_support._check_axis(axis, input.ndim)
+    axis = normalize_axis_index(axis, input.ndim)
     if (filter_size // 2 + origin < 0) or (filter_size // 2 + origin >=
                                            filter_size):
         raise ValueError('invalid origin')
@@ -1358,11 +1449,11 @@ def generic_filter1d(input, function, filter_size, axis=-1,
 def generic_filter(input, function, size=None, footprint=None,
                    output=None, mode="reflect", cval=0.0, origin=0,
                    extra_arguments=(), extra_keywords=None):
-    """Calculate a multi-dimensional filter using the given function.
+    """Calculate a multidimensional filter using the given function.
 
     At each element the provided function is called. The input values
     within the filter footprint at that element are passed to the function
-    as a 1D array of double values.
+    as a 1-D array of double values.
 
     Parameters
     ----------
@@ -1371,7 +1462,7 @@ def generic_filter(input, function, size=None, footprint=None,
         Function to apply at each element.
     %(size_foot)s
     %(output)s
-    %(mode_multiple)s
+    %(mode)s
     %(cval)s
     %(origin_multiple)s
     %(extra_arguments)s
