@@ -1,40 +1,78 @@
-"""
-An interior-point method for linear programming.
+"""Interior-point method for linear programming
+
+The *interior-point* method uses the primal-dual path following algorithm
+outlined in [1]_. This algorithm supports sparse constraint matrices and
+is typically faster than the simplex methods, especially for large, sparse
+problems. Note, however, that the solution returned may be slightly less
+accurate than those of the simplex methods and will not, in general,
+correspond with a vertex of the polytope defined by the constraints.
+
+    .. versionadded:: 1.0.0
+
+References
+----------
+.. [1] Andersen, Erling D., and Knud D. Andersen. "The MOSEK interior point
+       optimizer for linear programming: an implementation of the
+       homogeneous algorithm." High performance optimization. Springer US,
+       2000. 197-232.
 """
 # Author: Matt Haberland
 
-from __future__ import print_function, division, absolute_import
 import numpy as np
 import scipy as sp
 import scipy.sparse as sps
 from warnings import warn
 from scipy.linalg import LinAlgError
-from .optimize import OptimizeWarning, _check_unknown_options
+from .optimize import OptimizeWarning, OptimizeResult, _check_unknown_options
+from ._linprog_util import _postsolve
+has_umfpack = True
+has_cholmod = True
+try:
+    from sksparse.cholmod import cholesky as cholmod
+    from sksparse.cholmod import analyze as cholmod_analyze
+except ImportError:
+    has_cholmod = False
+try:
+    import scikits.umfpack  # test whether to use factorized
+except ImportError:
+    has_umfpack = False
 
 
-def _get_solver(sparse=False, lstsq=False, sym_pos=True, cholesky=True):
+def _get_solver(M, sparse=False, lstsq=False, sym_pos=True,
+                cholesky=True, permc_spec='MMD_AT_PLUS_A'):
     """
     Given solver options, return a handle to the appropriate linear system
     solver.
 
     Parameters
     ----------
-    sparse : bool
+    M : 2-D array
+        As defined in [4] Equation 8.31
+    sparse : bool (default = False)
         True if the system to be solved is sparse. This is typically set
         True when the original ``A_ub`` and ``A_eq`` arrays are sparse.
-    lstsq : bool
+    lstsq : bool (default = False)
         True if the system is ill-conditioned and/or (nearly) singular and
         thus a more robust least-squares solver is desired. This is sometimes
         needed as the solution is approached.
-    sym_pos : bool
+    sym_pos : bool (default = True)
         True if the system matrix is symmetric positive definite
         Sometimes this needs to be set false as the solution is approached,
         even when the system should be symmetric positive definite, due to
         numerical difficulties.
-    cholesky : bool
+    cholesky : bool (default = True)
         True if the system is to be solved by Cholesky, rather than LU,
         decomposition. This is typically faster unless the problem is very
         small or prone to numerical difficulties.
+    permc_spec : str (default = 'MMD_AT_PLUS_A')
+        Sparsity preservation strategy used by SuperLU. Acceptable values are:
+
+        - ``NATURAL``: natural ordering.
+        - ``MMD_ATA``: minimum degree ordering on the structure of A^T A.
+        - ``MMD_AT_PLUS_A``: minimum degree ordering on the structure of A^T+A.
+        - ``COLAMD``: approximate minimum degree column ordering.
+
+        See SuperLU documentation.
 
     Returns
     -------
@@ -42,50 +80,54 @@ def _get_solver(sparse=False, lstsq=False, sym_pos=True, cholesky=True):
         Handle to the appropriate solver function
 
     """
-    if sparse:
-        if lstsq or not(sym_pos):
-            def solve(M, r, sym_pos=False):
-                return sps.linalg.lsqr(M, r)[0]
-        else:
-            # this is not currently used; it is replaced by splu solve
-            # TODO: expose use of this as an option
-            def solve(M, r):
-                return sps.linalg.spsolve(M, r, permc_spec="MMD_AT_PLUS_A")
+    try:
+        if sparse:
+            if lstsq:
+                def solve(r, sym_pos=False):
+                    return sps.linalg.lsqr(M, r)[0]
+            elif cholesky:
+                try:
+                    # Will raise an exception in the first call,
+                    # or when the matrix changes due to a new problem
+                    _get_solver.cholmod_factor.cholesky_inplace(M)
+                except Exception:
+                    _get_solver.cholmod_factor = cholmod_analyze(M)
+                    _get_solver.cholmod_factor.cholesky_inplace(M)
+                solve = _get_solver.cholmod_factor
+            else:
+                if has_umfpack and sym_pos:
+                    solve = sps.linalg.factorized(M)
+                else:  # factorized doesn't pass permc_spec
+                    solve = sps.linalg.splu(M, permc_spec=permc_spec).solve
 
-    else:
-        if lstsq:  # sometimes necessary as solution is approached
-            def solve(M, r):
-                return sp.linalg.lstsq(M, r)[0]
-        elif cholesky:
-            solve = sp.linalg.cho_solve
         else:
-            # this seems to cache the matrix factorization, so solving
-            # with multiple right hand sides is much faster
-            def solve(M, r, sym_pos=sym_pos):
-                return sp.linalg.solve(M, r, sym_pos=sym_pos)
+            if lstsq:  # sometimes necessary as solution is approached
+                def solve(r):
+                    return sp.linalg.lstsq(M, r)[0]
+            elif cholesky:
+                L = sp.linalg.cho_factor(M)
 
+                def solve(r):
+                    return sp.linalg.cho_solve(L, r)
+            else:
+                # this seems to cache the matrix factorization, so solving
+                # with multiple right hand sides is much faster
+                def solve(r, sym_pos=sym_pos):
+                    return sp.linalg.solve(M, r, sym_pos=sym_pos)
+    # There are many things that can go wrong here, and it's hard to say
+    # what all of them are. It doesn't really matter: if the matrix can't be
+    # factorized, return None. get_solver will be called again with different
+    # inputs, and a new routine will try to factorize the matrix.
+    except KeyboardInterrupt:
+        raise
+    except Exception:
+        return None
     return solve
 
 
-def _get_delta(
-        A,
-        b,
-        c,
-        x,
-        y,
-        z,
-        tau,
-        kappa,
-        gamma,
-        eta,
-        sparse=False,
-        lstsq=False,
-        sym_pos=True,
-        cholesky=True,
-        pc=True,
-        ip=False,
-        permc_spec='MMD_AT_PLUS_A'
-    ):
+def _get_delta(A, b, c, x, y, z, tau, kappa, gamma, eta, sparse=False,
+               lstsq=False, sym_pos=True, cholesky=True, pc=True, ip=False,
+               permc_spec='MMD_AT_PLUS_A'):
     """
     Given standard form problem defined by ``A``, ``b``, and ``c``;
     current variable estimates ``x``, ``y``, ``z``, ``tau``, and ``kappa``;
@@ -150,12 +192,10 @@ def _get_delta(
            2000. 197-232.
 
     """
-
     if A.shape[0] == 0:
         # If there are no constraints, some solvers fail (understandably)
         # rather than returning empty solution. This gets the job done.
         sparse, lstsq, sym_pos, cholesky = False, False, True, False
-    solve = _get_solver(sparse, lstsq, sym_pos, cholesky)
     n_x = len(x)
 
     # [4] Equation 8.8
@@ -166,32 +206,12 @@ def _get_delta(
 
     #  Assemble M from [4] Equation 8.31
     Dinv = x / z
-    splu = False
-    if sparse and not lstsq:
-        # sparse requires Dinv to be diag matrix
-        M = A.dot(sps.diags(Dinv, 0, format="csc").dot(A.T))
-        try:
-            # TODO: should use linalg.factorized instead, but I don't have
-            #       umfpack and therefore cannot test its performance
-            solve = sps.linalg.splu(M, permc_spec=permc_spec).solve
-            splu = True
-        except Exception:
-            lstsq = True
-            solve = _get_solver(sparse, lstsq, sym_pos, cholesky)
-    else:
-        # dense does not; use broadcasting
-        M = A.dot(Dinv.reshape(-1, 1) * A.T)
 
-    # For some small problems, calling sp.linalg.solve w/ sym_pos = True
-    # may be faster. I am pretty certain it caches the factorization for
-    # multiple uses and checks the incoming matrix to see if it's the same as
-    # the one it already factorized. (I can't explain the speed otherwise.)
-    if cholesky:
-        try:
-            L = sp.linalg.cho_factor(M)
-        except Exception:
-            cholesky = False
-            solve = _get_solver(sparse, lstsq, sym_pos, cholesky)
+    if sparse:
+        M = A.dot(sps.diags(Dinv, 0, format="csc").dot(A.T))
+    else:
+        M = A.dot(Dinv.reshape(-1, 1) * A.T)
+    solve = _get_solver(M, sparse, lstsq, sym_pos, cholesky, permc_spec)
 
     # pc: "predictor-corrector" [4] Section 4.1
     # In development this option could be turned off
@@ -204,29 +224,20 @@ def _get_delta(
         # Reference [4] Eq. 8.6
         rhatp = eta(gamma) * r_P
         rhatd = eta(gamma) * r_D
-        rhatg = np.array(eta(gamma) * r_G).reshape((1,))
+        rhatg = eta(gamma) * r_G
 
         # Reference [4] Eq. 8.7
         rhatxs = gamma * mu - x * z
-        rhattk = np.array(gamma * mu - tau * kappa).reshape((1,))
+        rhattk = gamma * mu - tau * kappa
 
         if i == 1:
             if ip:  # if the correction is to get "initial point"
                 # Reference [4] Eq. 8.23
                 rhatxs = ((1 - alpha) * gamma * mu -
                           x * z - alpha**2 * d_x * d_z)
-                rhattk = np.array(
-                    (1 -
-                     alpha) *
-                    gamma *
-                    mu -
-                    tau *
-                    kappa -
-                    alpha**2 *
-                    d_tau *
-                    d_kappa).reshape(
-                    (1,
-                     ))
+                rhattk = ((1 - alpha) * gamma * mu -
+                    tau * kappa -
+                    alpha**2 * d_tau * d_kappa)
             else:  # if the correction is for "predictor-corrector"
                 # Reference [4] Eq. 8.13
                 rhatxs -= d_x * d_z
@@ -240,53 +251,59 @@ def _get_delta(
         # 3. scipy.linalg.solve w/ sym_pos = False, and if all else fails
         # 4. scipy.linalg.lstsq
         # For sparse systems, the order is:
-        # 1. scipy.sparse.linalg.splu
-        # 2. scipy.sparse.linalg.lsqr
-        # TODO: if umfpack is installed, use factorized instead of splu.
-        #       Can't do that now because factorized doesn't pass permc_spec
-        #       to splu if umfpack isn't installed. Also, umfpack not tested.
+        # 1. sksparse.cholmod.cholesky (if available)
+        # 2. scipy.sparse.linalg.factorized (if umfpack available)
+        # 3. scipy.sparse.linalg.splu
+        # 4. scipy.sparse.linalg.lsqr
         solved = False
         while(not solved):
             try:
-                solve_this = L if cholesky else M
                 # [4] Equation 8.28
-                p, q = _sym_solve(Dinv, solve_this, A, c, b, solve, splu)
+                p, q = _sym_solve(Dinv, A, c, b, solve)
                 # [4] Equation 8.29
-                u, v = _sym_solve(Dinv, solve_this, A, rhatd -
-                                  (1 / x) * rhatxs, rhatp, solve, splu)
+                u, v = _sym_solve(Dinv, A, rhatd -
+                                  (1 / x) * rhatxs, rhatp, solve)
                 if np.any(np.isnan(p)) or np.any(np.isnan(q)):
                     raise LinAlgError
                 solved = True
-            except (LinAlgError, ValueError) as e:
+            except (LinAlgError, ValueError, TypeError) as e:
                 # Usually this doesn't happen. If it does, it happens when
                 # there are redundant constraints or when approaching the
                 # solution. If so, change solver.
-                cholesky = False
-                if not lstsq:
-                    if sym_pos:
-                        warn(
-                            "Solving system with option 'sym_pos':True "
-                            "failed. It is normal for this to happen "
-                            "occasionally, especially as the solution is "
-                            "approached. However, if you see this frequently, "
-                            "consider setting option 'sym_pos' to False.",
-                            OptimizeWarning)
-                        sym_pos = False
-                    else:
-                        warn(
-                            "Solving system with option 'sym_pos':False "
-                            "failed. This may happen occasionally, "
-                            "especially as the solution is "
-                            "approached. However, if you see this frequently, "
-                            "your problem may be numerically challenging. "
-                            "If you cannot improve the formulation, consider "
-                            "setting 'lstsq' to True. Consider also setting "
-                            "`presolve` to True, if it is not already.",
-                            OptimizeWarning)
-                        lstsq = True
+                if cholesky:
+                    cholesky = False
+                    warn(
+                        "Solving system with option 'cholesky':True "
+                        "failed. It is normal for this to happen "
+                        "occasionally, especially as the solution is "
+                        "approached. However, if you see this frequently, "
+                        "consider setting option 'cholesky' to False.",
+                        OptimizeWarning, stacklevel=5)
+                elif sym_pos:
+                    sym_pos = False
+                    warn(
+                        "Solving system with option 'sym_pos':True "
+                        "failed. It is normal for this to happen "
+                        "occasionally, especially as the solution is "
+                        "approached. However, if you see this frequently, "
+                        "consider setting option 'sym_pos' to False.",
+                        OptimizeWarning, stacklevel=5)
+                elif not lstsq:
+                    lstsq = True
+                    warn(
+                        "Solving system with option 'sym_pos':False "
+                        "failed. This may happen occasionally, "
+                        "especially as the solution is "
+                        "approached. However, if you see this frequently, "
+                        "your problem may be numerically challenging. "
+                        "If you cannot improve the formulation, consider "
+                        "setting 'lstsq' to True. Consider also setting "
+                        "`presolve` to True, if it is not already.",
+                        OptimizeWarning, stacklevel=5)
                 else:
                     raise e
-                solve = _get_solver(sparse, lstsq, sym_pos)
+                solve = _get_solver(M, sparse, lstsq, sym_pos,
+                                    cholesky, permc_spec)
         # [4] Results after 8.29
         d_tau = ((rhatg + 1 / tau * rhattk - (-c.dot(u) + b.dot(v))) /
                  (1 / tau * kappa + (-c.dot(p) + b.dot(q))))
@@ -309,7 +326,7 @@ def _get_delta(
     return d_x, d_y, d_z, d_tau, d_kappa
 
 
-def _sym_solve(Dinv, M, A, r1, r2, solve, splu=False):
+def _sym_solve(Dinv, A, r1, r2, solve):
     """
     An implementation of [4] equation 8.31 and 8.32
 
@@ -323,10 +340,7 @@ def _sym_solve(Dinv, M, A, r1, r2, solve, splu=False):
     """
     # [4] 8.31
     r = r2 + A.dot(Dinv * r1)
-    if splu:
-        v = solve(r)
-    else:
-        v = solve(M, r)
+    v = solve(r)
     # [4] 8.32
     u = Dinv * (A.T.dot(v) - r1)
     return u, v
@@ -525,16 +539,16 @@ def _display_iter(rho_p, rho_d, rho_g, alpha, rho_mu, obj, header=False):
     # no clue why this works
     fmt = '{0:<20.13}{1:<20.13}{2:<20.13}{3:<17.13}{4:<20.13}{5:<20.13}'
     print(fmt.format(
-        rho_p,
-        rho_d,
-        rho_g,
-        alpha,
-        rho_mu,
-        obj))
+        float(rho_p),
+        float(rho_d),
+        float(rho_g),
+        alpha if isinstance(alpha, str) else float(alpha),
+        float(rho_mu),
+        float(obj)))
 
 
-def _ip_hsd(A, b, c, c0, alpha0, beta, maxiter, disp, tol,
-            sparse, lstsq, sym_pos, cholesky, pc, ip, permc_spec):
+def _ip_hsd(A, b, c, c0, alpha0, beta, maxiter, disp, tol, sparse, lstsq,
+            sym_pos, cholesky, pc, ip, permc_spec, callback, postsolve_args):
     r"""
     Solve a linear programming problem in standard form:
 
@@ -551,13 +565,13 @@ def _ip_hsd(A, b, c, c0, alpha0, beta, maxiter, disp, tol,
 
     Parameters
     ----------
-    A : 2D array
-        2D array such that ``A @ x``, gives the values of the equality
+    A : 2-D array
+        2-D array such that ``A @ x``, gives the values of the equality
         constraints at ``x``.
-    b : 1D array
-        1D array of values representing the RHS of each equality constraint
+    b : 1-D array
+        1-D array of values representing the RHS of each equality constraint
         (row) in ``A`` (for standard form problem).
-    c : 1D array
+    c : 1-D array
         Coefficients of the linear objective function to be minimized (for
         standard form problem).
     c0 : float
@@ -613,6 +627,39 @@ def _ip_hsd(A, b, c, c0, alpha0, beta, maxiter, disp, tol,
         interior point algorithm; test different values to determine which
         performs best for your problem. For more information, refer to
         ``scipy.sparse.linalg.splu``.
+    callback : callable, optional
+        If a callback function is provided, it will be called within each
+        iteration of the algorithm. The callback function must accept a single
+        `scipy.optimize.OptimizeResult` consisting of the following fields:
+
+            x : 1-D array
+                Current solution vector
+            fun : float
+                Current value of the objective function
+            success : bool
+                True only when an algorithm has completed successfully,
+                so this is always False as the callback function is called
+                only while the algorithm is still iterating.
+            slack : 1-D array
+                The values of the slack variables. Each slack variable
+                corresponds to an inequality constraint. If the slack is zero,
+                the corresponding constraint is active.
+            con : 1-D array
+                The (nominally zero) residuals of the equality constraints,
+                that is, ``b - A_eq @ x``
+            phase : int
+                The phase of the algorithm being executed. This is always
+                1 for the interior-point method because it has only one phase.
+            status : int
+                For revised simplex, this is always 0 because if a different
+                status is detected, the algorithm terminates.
+            nit : int
+                The number of iterations performed.
+            message : str
+                A string descriptor of the exit status of the optimization.
+    postsolve_args : tuple
+        Data needed by _postsolve to convert the solution to the standard-form
+        problem into the solution to the original problem.
 
     Returns
     -------
@@ -660,6 +707,14 @@ def _ip_hsd(A, b, c, c0, alpha0, beta, maxiter, disp, tol,
 
     if disp:
         _display_iter(rho_p, rho_d, rho_g, "-", rho_mu, obj, header=True)
+    if callback is not None:
+        x_o, fun, slack, con, _ = _postsolve(x/tau, postsolve_args,
+                                                tol=tol)
+        res = OptimizeResult({'x': x_o, 'fun': fun, 'slack': slack,
+                              'con': con, 'nit': iteration, 'phase': 1,
+                              'complete': False, 'status': 0,
+                              'message': "", 'success': False})
+        callback(res)
 
     status = 0
     message = "Optimization terminated successfully."
@@ -732,7 +787,15 @@ def _ip_hsd(A, b, c, c0, alpha0, beta, maxiter, disp, tol,
         go = rho_p > tol or rho_d > tol or rho_A > tol
 
         if disp:
-            _display_iter(rho_p, rho_d, rho_g, alpha, float(rho_mu), obj)
+            _display_iter(rho_p, rho_d, rho_g, alpha, rho_mu, obj)
+        if callback is not None:
+            x_o, fun, slack, con, _ = _postsolve(x/tau, postsolve_args,
+                                                    tol=tol)
+            res = OptimizeResult({'x': x_o, 'fun': fun, 'slack': slack,
+                                  'con': con, 'nit': iteration, 'phase': 1,
+                                  'complete': False, 'status': 0,
+                                  'message': "", 'success': False})
+            callback(res)
 
         # [4] 4.5
         inf1 = (rho_p < tol and rho_d < tol and rho_g < tol and tau < tol *
@@ -756,25 +819,10 @@ def _ip_hsd(A, b, c, c0, alpha0, beta, maxiter, disp, tol,
     return x_hat, status, message, iteration
 
 
-def _linprog_ip(
-        c,
-        c0=0,
-        A=None,
-        b=None,
-        callback=None,
-        alpha0=.99995,
-        beta=0.1,
-        maxiter=1000,
-        disp=False,
-        tol=1e-8,
-        sparse=False,
-        lstsq=False,
-        sym_pos=True,
-        cholesky=None,
-        pc=True,
-        ip=False,
-        permc_spec='MMD_AT_PLUS_A',
-        **unknown_options):
+def _linprog_ip(c, c0, A, b, callback, postsolve_args, maxiter=1000, tol=1e-8,
+                disp=False, alpha0=.99995, beta=0.1, sparse=False, lstsq=False,
+                sym_pos=True, cholesky=None, pc=True, ip=False,
+                permc_spec='MMD_AT_PLUS_A', **unknown_options):
     r"""
     Minimize a linear objective function subject to linear
     equality and non-negativity constraints using the interior point method
@@ -792,28 +840,33 @@ def _linprog_ip(
 
     Parameters
     ----------
-    c : 1D array
+    c : 1-D array
         Coefficients of the linear objective function to be minimized.
     c0 : float
         Constant term in objective function due to fixed (and eliminated)
         variables. (Purely for display.)
-    A : 2D array
-        2D array such that ``A @ x``, gives the values of the equality
+    A : 2-D array
+        2-D array such that ``A @ x``, gives the values of the equality
         constraints at ``x``.
-    b : 1D array
-        1D array of values representing the right hand side of each equality
+    b : 1-D array
+        1-D array of values representing the right hand side of each equality
         constraint (row) in ``A``.
+    callback : callable, optional
+        Callback function to be executed once per iteration.
+    postsolve_args : tuple
+        Data needed by _postsolve to convert the solution to the standard-form
+        problem into the solution to the original problem.
 
     Options
     -------
     maxiter : int (default = 1000)
         The maximum number of iterations of the algorithm.
-    disp : bool (default = False)
-        Set to ``True`` if indicators of optimization status are to be printed
-        to the console each iteration.
     tol : float (default = 1e-8)
         Termination tolerance to be used for all termination criteria;
         see [4]_ Section 4.5.
+    disp : bool (default = False)
+        Set to ``True`` if indicators of optimization status are to be printed
+        to the console each iteration.
     alpha0 : float (default = 0.99995)
         The maximal step size for Mehrota's predictor-corrector search
         direction; see :math:`\beta_{3}` of [4]_ Table 8.1.
@@ -841,7 +894,7 @@ def _linprog_ip(
     cholesky : bool (default = True)
         Set to ``True`` if the normal equations are to be solved by explicit
         Cholesky decomposition followed by explicit forward/backward
-        substitution. This is typically faster for moderate, dense problems
+        substitution. This is typically faster for problems
         that are numerically well-behaved.
     pc : bool (default = True)
         Leave ``True`` if the predictor-corrector method of Mehrota is to be
@@ -852,7 +905,8 @@ def _linprog_ip(
         depends on the problem.
     permc_spec : str (default = 'MMD_AT_PLUS_A')
         (Has effect only with ``sparse = True``, ``lstsq = False``, ``sym_pos =
-        True``.) A matrix is factorized in each iteration of the algorithm.
+        True``, and no SuiteSparse.)
+        A matrix is factorized in each iteration of the algorithm.
         This option specifies how to permute the columns of the matrix for
         sparsity preservation. Acceptable values are:
 
@@ -865,10 +919,14 @@ def _linprog_ip(
         interior point algorithm; test different values to determine which
         performs best for your problem. For more information, refer to
         ``scipy.sparse.linalg.splu``.
+    unknown_options : dict
+        Optional arguments not used by this particular solver. If
+        `unknown_options` is non-empty a warning is issued listing all
+        unused options.
 
     Returns
     -------
-    x : 1D array
+    x : 1-D array
         Solution vector.
     status : int
         An integer representing the exit status of the optimization::
@@ -887,7 +945,7 @@ def _linprog_ip(
     Notes
     -----
     This method implements the algorithm outlined in [4]_ with ideas from [8]_
-    and a structure inspired by the simpler methods of [6]_ and [4]_.
+    and a structure inspired by the simpler methods of [6]_.
 
     The primal-dual path following method begins with initial 'guesses' of
     the primal and dual variables of the standard form problem and iteratively
@@ -914,33 +972,38 @@ def _linprog_ip(
     matrices involved are symmetric positive definite, so Cholesky
     decomposition can be used rather than the more expensive LU factorization.
 
-    With the default ``cholesky=True``, this is accomplished using
-    ``scipy.linalg.cho_factor`` followed by forward/backward substitutions
-    via ``scipy.linalg.cho_solve``. With ``cholesky=False`` and
-    ``sym_pos=True``, Cholesky decomposition is performed instead by
-    ``scipy.linalg.solve``. Based on speed tests, this also appears to retain
-    the Cholesky decomposition of the matrix for later use, which is beneficial
-    as the same system is solved four times with different right hand sides
-    in each iteration of the algorithm.
+    With default options, the solver used to perform the factorization depends
+    on third-party software availability and the conditioning of the problem.
 
-    In problems with redundancy (e.g. if presolve is turned off with option
-    ``presolve=False``) or if the matrices become ill-conditioned (e.g. as the
-    solution is approached and some decision variables approach zero),
-    Cholesky decomposition can fail. Should this occur, successively more
-    robust solvers (``scipy.linalg.solve`` with ``sym_pos=False`` then
-    ``scipy.linalg.lstsq``) are tried, at the cost of computational efficiency.
-    These solvers can be used from the outset by setting the options
-    ``sym_pos=False`` and ``lstsq=True``, respectively.
+    For dense problems, solvers are tried in the following order:
 
-    Note that with the option ``sparse=True``, the normal equations are solved
-    using ``scipy.sparse.linalg.spsolve``. Unfortunately, this uses the more
-    expensive LU decomposition from the outset, but for large, sparse problems,
-    the use of sparse linear algebra techniques improves the solve speed
-    despite the use of LU rather than Cholesky decomposition. A simple
-    improvement would be to use the sparse Cholesky decomposition of
-    ``CHOLMOD`` via ``scikit-sparse`` when available.
+    1. ``scipy.linalg.cho_factor``
 
-    Other potential improvements for combatting issues associated with dense
+    2. ``scipy.linalg.solve`` with option ``sym_pos=True``
+
+    3. ``scipy.linalg.solve`` with option ``sym_pos=False``
+
+    4. ``scipy.linalg.lstsq``
+
+    For sparse problems:
+
+    1. ``sksparse.cholmod.cholesky`` (if scikit-sparse and SuiteSparse are installed)
+
+    2. ``scipy.sparse.linalg.factorized`` (if scikit-umfpack and SuiteSparse are installed)
+
+    3. ``scipy.sparse.linalg.splu`` (which uses SuperLU distributed with SciPy)
+
+    4. ``scipy.sparse.linalg.lsqr``
+
+    If the solver fails for any reason, successively more robust (but slower)
+    solvers are attempted in the order indicated. Attempting, failing, and
+    re-starting factorization can be time consuming, so if the problem is
+    numerically challenging, options can be set to  bypass solvers that are
+    failing. Setting ``cholesky=False`` skips to solver 2,
+    ``sym_pos=False`` skips to solver 3, and ``lstsq=True`` skips
+    to solver 4 for both sparse and dense problems.
+
+    Potential improvements for combatting issues associated with dense
     columns in otherwise sparse problems are outlined in [4]_ Section 5.3 and
     [10]_ Section 4.1-4.2; the latter also discusses the alleviation of
     accuracy issues associated with the substitution approach to free
@@ -1015,41 +1078,32 @@ def _linprog_ip(
     """
 
     _check_unknown_options(unknown_options)
-    if callback is not None:
-        raise NotImplementedError("method 'interior-point' does not support "
-                                  "callback functions.")
 
     # These should be warnings, not errors
+    if (cholesky or cholesky is None) and sparse and not has_cholmod:
+        if cholesky:
+            warn("Sparse cholesky is only available with scikit-sparse. "
+                 "Setting `cholesky = False`",
+                 OptimizeWarning, stacklevel=3)
+        cholesky = False
+
     if sparse and lstsq:
-        warn("Invalid option combination 'sparse':True "
-             "and 'lstsq':True; Sparse least squares is not recommended.",
-             OptimizeWarning)
-
-    if sparse and not sym_pos:
-        warn("Invalid option combination 'sparse':True "
-             "and 'sym_pos':False; the effect is the same as sparse least "
-             "squares, which is not recommended.",
-             OptimizeWarning)
-
-    if sparse and cholesky:
-        # Cholesky decomposition is not available for sparse problems
-        warn("Invalid option combination 'sparse':True "
-             "and 'cholesky':True; sparse Colesky decomposition is not "
-             "available.",
-             OptimizeWarning)
+        warn("Option combination 'sparse':True and 'lstsq':True "
+             "is not recommended.",
+             OptimizeWarning, stacklevel=3)
 
     if lstsq and cholesky:
         warn("Invalid option combination 'lstsq':True "
              "and 'cholesky':True; option 'cholesky' has no effect when "
              "'lstsq' is set True.",
-             OptimizeWarning)
+             OptimizeWarning, stacklevel=3)
 
     valid_permc_spec = ('NATURAL', 'MMD_ATA', 'MMD_AT_PLUS_A', 'COLAMD')
     if permc_spec.upper() not in valid_permc_spec:
         warn("Invalid permc_spec option: '" + str(permc_spec) + "'. "
              "Acceptable values are 'NATURAL', 'MMD_ATA', 'MMD_AT_PLUS_A', "
              "and 'COLAMD'. Reverting to default.",
-             OptimizeWarning)
+             OptimizeWarning, stacklevel=3)
         permc_spec = 'MMD_AT_PLUS_A'
 
     # This can be an error
@@ -1059,11 +1113,12 @@ def _linprog_ip(
             "and 'cholesky':True: Cholesky decomposition is only possible "
             "for symmetric positive definite matrices.")
 
-    cholesky = cholesky is None and sym_pos and not sparse and not lstsq
+    cholesky = cholesky or (cholesky is None and sym_pos and not lstsq)
 
     x, status, message, iteration = _ip_hsd(A, b, c, c0, alpha0, beta,
                                             maxiter, disp, tol, sparse,
                                             lstsq, sym_pos, cholesky,
-                                            pc, ip, permc_spec)
+                                            pc, ip, permc_spec, callback,
+                                            postsolve_args)
 
     return x, status, message, iteration
