@@ -8,25 +8,24 @@ Sparse matrix functions
 #          Jake Vanderplas, August 2012 (Sparse Updates)
 #
 
-from __future__ import division, print_function, absolute_import
-
 __all__ = ['expm', 'inv']
 
-import math
-
 import numpy as np
 
 import scipy.special
+from scipy._lib._util import float_factorial
 from scipy.linalg.basic import solve, solve_triangular
 
 from scipy.sparse.base import isspmatrix
-from scipy.sparse.construct import eye as speye
 from scipy.sparse.linalg import spsolve
+from scipy.sparse.sputils import is_pydata_spmatrix
 
 import scipy.sparse
 import scipy.sparse.linalg
 from scipy.sparse.linalg.interface import LinearOperator
 
+from ._expm_multiply import _ident_like, _exact_1_norm as _onenorm
+
 
 UPPER_TRIANGULAR = 'upper_triangular'
 
@@ -47,7 +46,7 @@ def inv(A):
 
     Notes
     -----
-    This computes the sparse inverse of `A`.  If the inverse of `A` is expected
+    This computes the sparse inverse of `A`. If the inverse of `A` is expected
     to be non-sparse, it will likely be faster to convert `A` to dense and use
     scipy.linalg.inv.
 
@@ -71,10 +70,10 @@ def inv(A):
 
     """
     #check input
-    if not scipy.sparse.isspmatrix(A):
+    if not (scipy.sparse.isspmatrix(A) or is_pydata_spmatrix(A)):
         raise TypeError('Input must be a sparse matrix')
 
-    I = speye(A.shape[0], A.shape[1], dtype=A.dtype, format=A.format)
+    I = _ident_like(A)
     Ainv = spsolve(A, I)
     return Ainv
 
@@ -112,25 +111,6 @@ def _onenorm_matrix_power_nnm(A, p):
     return np.max(v)
 
 
-def _onenorm(A):
-    # A compatibility function which should eventually disappear.
-    # This is copypasted from expm_action.
-    if scipy.sparse.isspmatrix(A):
-        return max(abs(A).sum(axis=0).flat)
-    else:
-        return np.linalg.norm(A, 1)
-
-
-def _ident_like(A):
-    # A compatibility function which should eventually disappear.
-    # This is copypasted from expm_action.
-    if scipy.sparse.isspmatrix(A):
-        return scipy.sparse.construct.eye(A.shape[0], A.shape[1],
-                dtype=A.dtype, format=A.format)
-    else:
-        return np.eye(A.shape[0], A.shape[1], dtype=A.dtype)
-
-
 def _is_upper_triangular(A):
     # This function could possibly be of wider interest.
     if isspmatrix(A):
@@ -138,6 +118,10 @@ def _is_upper_triangular(A):
         # Check structural upper triangularity,
         # then coincidental upper triangularity if needed.
         return lower_part.nnz == 0 or lower_part.count_nonzero() == 0
+    elif is_pydata_spmatrix(A):
+        import sparse
+        lower_part = sparse.tril(A, -1)
+        return lower_part.nnz == 0
     else:
         return not np.tril(A, -1).any()
 
@@ -170,7 +154,8 @@ def _smart_matrix_product(A, B, alpha=None, structure=None):
         raise ValueError('expected B to be a rectangular matrix')
     f = None
     if structure == UPPER_TRIANGULAR:
-        if not isspmatrix(A) and not isspmatrix(B):
+        if (not isspmatrix(A) and not isspmatrix(B)
+                and not is_pydata_spmatrix(A) and not is_pydata_spmatrix(B)):
             f, = scipy.linalg.get_blas_funcs(('trmm',), (A, B))
     if f is not None:
         if alpha is None:
@@ -616,19 +601,27 @@ def _expm(A, use_exact_onenorm):
     if len(A.shape) != 2 or A.shape[0] != A.shape[1]:
         raise ValueError('expected a square matrix')
 
+    # gracefully handle size-0 input,
+    # carefully handling sparse scenario
+    if A.shape == (0, 0):
+        out = np.zeros([0, 0], dtype=A.dtype)
+        if isspmatrix(A) or is_pydata_spmatrix(A):
+            return A.__class__(out)
+        return out
+
     # Trivial case
     if A.shape == (1, 1):
         out = [[np.exp(A[0, 0])]]
 
         # Avoid indiscriminate casting to ndarray to
         # allow for sparse or other strange arrays
-        if isspmatrix(A):
+        if isspmatrix(A) or is_pydata_spmatrix(A):
             return A.__class__(out)
 
         return np.array(out)
 
     # Ensure input is of float type, to avoid integer overflows etc.
-    if ((isinstance(A, np.ndarray) or isspmatrix(A))
+    if ((isinstance(A, np.ndarray) or isspmatrix(A) or is_pydata_spmatrix(A))
             and not np.issubdtype(A.dtype, np.inexact)):
         A = A.astype(float)
 
@@ -710,7 +703,7 @@ def _solve_P_Q(U, V, structure=None):
     """
     P = U + V
     Q = -U + V
-    if isspmatrix(U):
+    if isspmatrix(U) or is_pydata_spmatrix(U):
         return spsolve(Q, P)
     elif structure is None:
         return solve(Q, P)
@@ -720,9 +713,9 @@ def _solve_P_Q(U, V, structure=None):
         raise ValueError('unsupported matrix structure: ' + str(structure))
 
 
-def _sinch(x):
+def _exp_sinch(a, x):
     """
-    Stably evaluate sinch.
+    Stably evaluate exp(a)*sinh(x)/x
 
     Notes
     -----
@@ -744,11 +737,11 @@ def _sinch(x):
     # How small is small? I am using the point where the relative error
     # of the approximation is less than 1e-14.
     # If x is large then directly evaluate sinh(x) / x.
-    x2 = x*x
     if abs(x) < 0.0135:
-        return 1 + (x2/6.)*(1 + (x2/20.)*(1 + (x2/42.)))
+        x2 = x*x
+        return np.exp(a) * (1 + (x2/6.)*(1 + (x2/20.)*(1 + (x2/42.))))
     else:
-        return np.sinh(x) / x
+        return (np.exp(a + x) - np.exp(a - x)) / (2*x)
 
 
 def _eq_10_42(lam_1, lam_2, t_12):
@@ -772,7 +765,7 @@ def _eq_10_42(lam_1, lam_2, t_12):
     # will apparently work around the cancellation.
     a = 0.5 * (lam_1 + lam_2)
     b = 0.5 * (lam_1 - lam_2)
-    return t_12 * np.exp(a) * _sinch(b)
+    return t_12 * _exp_sinch(a, b)
 
 
 def _fragment_2_1(X, T, s):
@@ -844,19 +837,17 @@ def _ell(A, m):
     if len(A.shape) != 2 or A.shape[0] != A.shape[1]:
         raise ValueError('expected A to be like a square matrix')
 
-    p = 2*m + 1
-
     # The c_i are explained in (2.2) and (2.6) of the 2005 expm paper.
     # They are coefficients of terms of a generating function series expansion.
-    choose_2p_p = scipy.special.comb(2*p, p, exact=True)
-    abs_c_recip = float(choose_2p_p * math.factorial(2*p + 1))
+    choose_2m_m = scipy.special.comb(2*m, m, exact=True)
+    abs_c_recip = float(choose_2m_m) * float_factorial(2*m + 1)
 
     # This is explained after Eq. (1.2) of the 2009 expm paper.
     # It is the "unit roundoff" of IEEE double precision arithmetic.
     u = 2**-53
 
     # Compute the one-norm of matrix power p of abs(A).
-    A_abs_onenorm = _onenorm_matrix_power_nnm(abs(A), p)
+    A_abs_onenorm = _onenorm_matrix_power_nnm(abs(A), 2*m + 1)
 
     # Treat zero norm as a special case.
     if not A_abs_onenorm:
