@@ -1,5 +1,3 @@
-from __future__ import division, print_function, absolute_import
-
 import pytest
 
 import numpy as np
@@ -67,16 +65,16 @@ def test_zero_norms_from_quat():
         Rotation.from_quat(x)
 
 
-def test_as_dcm_single_1d_quaternion():
+def test_as_matrix_single_1d_quaternion():
     quat = [0, 0, 0, 1]
-    mat = Rotation.from_quat(quat).as_dcm()
+    mat = Rotation.from_quat(quat).as_matrix()
     # mat.shape == (3,3) due to 1d input
     assert_array_almost_equal(mat, np.eye(3))
 
 
-def test_as_dcm_single_2d_quaternion():
+def test_as_matrix_single_2d_quaternion():
     quat = [[0, 0, 1, 1]]
-    mat = Rotation.from_quat(quat).as_dcm()
+    mat = Rotation.from_quat(quat).as_matrix()
     assert_equal(mat.shape, (1, 3, 3))
     expected_mat = np.array([
         [0, -1, 0],
@@ -86,14 +84,14 @@ def test_as_dcm_single_2d_quaternion():
     assert_array_almost_equal(mat[0], expected_mat)
 
 
-def test_as_dcm_from_square_input():
+def test_as_matrix_from_square_input():
     quats = [
             [0, 0, 1, 1],
             [0, 1, 0, 1],
             [0, 0, 0, 1],
             [0, 0, 0, -1]
             ]
-    mat = Rotation.from_quat(quats).as_dcm()
+    mat = Rotation.from_quat(quats).as_matrix()
     assert_equal(mat.shape, (4, 3, 3))
 
     expected0 = np.array([
@@ -114,13 +112,13 @@ def test_as_dcm_from_square_input():
     assert_array_almost_equal(mat[3], np.eye(3))
 
 
-def test_as_dcm_from_generic_input():
+def test_as_matrix_from_generic_input():
     quats = [
             [0, 0, 1, 1],
             [0, 1, 0, 1],
             [1, 2, 3, 4]
             ]
-    mat = Rotation.from_quat(quats).as_dcm()
+    mat = Rotation.from_quat(quats).as_matrix()
     assert_equal(mat.shape, (3, 3, 3))
 
     expected0 = np.array([
@@ -145,57 +143,57 @@ def test_as_dcm_from_generic_input():
     assert_array_almost_equal(mat[2], expected2)
 
 
-def test_from_single_2d_dcm():
-    dcm = [
+def test_from_single_2d_matrix():
+    mat = [
             [0, 0, 1],
             [1, 0, 0],
             [0, 1, 0]
             ]
     expected_quat = [0.5, 0.5, 0.5, 0.5]
     assert_array_almost_equal(
-            Rotation.from_dcm(dcm).as_quat(),
+            Rotation.from_matrix(mat).as_quat(),
             expected_quat)
 
 
-def test_from_single_3d_dcm():
-    dcm = np.array([
+def test_from_single_3d_matrix():
+    mat = np.array([
         [0, 0, 1],
         [1, 0, 0],
         [0, 1, 0]
         ]).reshape((1, 3, 3))
     expected_quat = np.array([0.5, 0.5, 0.5, 0.5]).reshape((1, 4))
     assert_array_almost_equal(
-            Rotation.from_dcm(dcm).as_quat(),
+            Rotation.from_matrix(mat).as_quat(),
             expected_quat)
 
 
-def test_from_dcm_calculation():
+def test_from_matrix_calculation():
     expected_quat = np.array([1, 1, 6, 1]) / np.sqrt(39)
-    dcm = np.array([
+    mat = np.array([
             [-0.8974359, -0.2564103, 0.3589744],
             [0.3589744, -0.8974359, 0.2564103],
             [0.2564103, 0.3589744, 0.8974359]
             ])
     assert_array_almost_equal(
-            Rotation.from_dcm(dcm).as_quat(),
+            Rotation.from_matrix(mat).as_quat(),
             expected_quat)
     assert_array_almost_equal(
-            Rotation.from_dcm(dcm.reshape((1, 3, 3))).as_quat(),
+            Rotation.from_matrix(mat.reshape((1, 3, 3))).as_quat(),
             expected_quat.reshape((1, 4)))
 
 
-def test_dcm_calculation_pipeline():
-    dcm = special_ortho_group.rvs(3, size=10, random_state=0)
-    assert_array_almost_equal(Rotation.from_dcm(dcm).as_dcm(), dcm)
+def test_matrix_calculation_pipeline():
+    mat = special_ortho_group.rvs(3, size=10, random_state=0)
+    assert_array_almost_equal(Rotation.from_matrix(mat).as_matrix(), mat)
 
 
-def test_from_dcm_ortho_output():
+def test_from_matrix_ortho_output():
     np.random.seed(0)
-    dcm = np.random.random((100, 3, 3))
-    ortho_dcm = Rotation.from_dcm(dcm).as_dcm()
+    mat = np.random.random((100, 3, 3))
+    ortho_mat = Rotation.from_matrix(mat).as_matrix()
 
-    mult_result = np.einsum('...ij,...jk->...ik', ortho_dcm,
-                            ortho_dcm.transpose((0, 2, 1)))
+    mult_result = np.einsum('...ij,...jk->...ik', ortho_mat,
+                            ortho_mat.transpose((0, 2, 1)))
 
     eye3d = np.zeros((100, 3, 3))
     for i in range(3):
@@ -324,8 +322,8 @@ def test_from_euler_single_rotation():
 
 
 def test_single_intrinsic_extrinsic_rotation():
-    extrinsic = Rotation.from_euler('z', 90, degrees=True).as_dcm()
-    intrinsic = Rotation.from_euler('Z', 90, degrees=True).as_dcm()
+    extrinsic = Rotation.from_euler('z', 90, degrees=True).as_matrix()
+    intrinsic = Rotation.from_euler('Z', 90, degrees=True).as_matrix()
     assert_allclose(extrinsic, intrinsic)
 
 
@@ -341,13 +339,13 @@ def test_from_euler_rotation_order():
 
 def test_from_euler_elementary_extrinsic_rotation():
     # Simple test to check if extrinsic rotations are implemented correctly
-    dcm = Rotation.from_euler('zx', [90, 90], degrees=True).as_dcm()
-    expected_dcm = np.array([
+    mat = Rotation.from_euler('zx', [90, 90], degrees=True).as_matrix()
+    expected_mat = np.array([
         [0, -1, 0],
         [0, 0, -1],
         [1, 0, 0]
     ])
-    assert_array_almost_equal(dcm, expected_dcm)
+    assert_array_almost_equal(mat, expected_mat)
 
 
 def test_from_euler_intrinsic_rotation_312():
@@ -356,21 +354,21 @@ def test_from_euler_intrinsic_rotation_312():
         [30, 60, 30],
         [45, 30, 60]
         ]
-    dcm = Rotation.from_euler('ZXY', angles, degrees=True).as_dcm()
+    mat = Rotation.from_euler('ZXY', angles, degrees=True).as_matrix()
 
-    assert_array_almost_equal(dcm[0], np.array([
+    assert_array_almost_equal(mat[0], np.array([
         [0.3061862, -0.2500000, 0.9185587],
         [0.8838835, 0.4330127, -0.1767767],
         [-0.3535534, 0.8660254, 0.3535534]
     ]))
 
-    assert_array_almost_equal(dcm[1], np.array([
+    assert_array_almost_equal(mat[1], np.array([
         [0.5334936, -0.2500000, 0.8080127],
         [0.8080127, 0.4330127, -0.3995191],
         [-0.2500000, 0.8660254, 0.4330127]
     ]))
 
-    assert_array_almost_equal(dcm[2], np.array([
+    assert_array_almost_equal(mat[2], np.array([
         [0.0473672, -0.6123725, 0.7891491],
         [0.6597396, 0.6123725, 0.4355958],
         [-0.7500000, 0.5000000, 0.4330127]
@@ -383,21 +381,21 @@ def test_from_euler_intrinsic_rotation_313():
         [30, 60, 30],
         [45, 30, 60]
         ]
-    dcm = Rotation.from_euler('ZXZ', angles, degrees=True).as_dcm()
+    mat = Rotation.from_euler('ZXZ', angles, degrees=True).as_matrix()
 
-    assert_array_almost_equal(dcm[0], np.array([
+    assert_array_almost_equal(mat[0], np.array([
         [0.43559574, -0.78914913, 0.4330127],
         [0.65973961, -0.04736717, -0.750000],
         [0.61237244, 0.61237244, 0.500000]
     ]))
 
-    assert_array_almost_equal(dcm[1], np.array([
+    assert_array_almost_equal(mat[1], np.array([
         [0.6250000, -0.64951905, 0.4330127],
         [0.64951905, 0.1250000, -0.750000],
         [0.4330127, 0.750000, 0.500000]
     ]))
 
-    assert_array_almost_equal(dcm[2], np.array([
+    assert_array_almost_equal(mat[2], np.array([
         [-0.1767767, -0.91855865, 0.35355339],
         [0.88388348, -0.30618622, -0.35355339],
         [0.4330127, 0.25000000, 0.8660254]
@@ -410,21 +408,21 @@ def test_from_euler_extrinsic_rotation_312():
         [30, 60, 30],
         [45, 30, 60]
         ]
-    dcm = Rotation.from_euler('zxy', angles, degrees=True).as_dcm()
+    mat = Rotation.from_euler('zxy', angles, degrees=True).as_matrix()
 
-    assert_array_almost_equal(dcm[0], np.array([
+    assert_array_almost_equal(mat[0], np.array([
         [0.91855865, 0.1767767, 0.35355339],
         [0.25000000, 0.4330127, -0.8660254],
         [-0.30618622, 0.88388348, 0.35355339]
     ]))
 
-    assert_array_almost_equal(dcm[1], np.array([
+    assert_array_almost_equal(mat[1], np.array([
         [0.96650635, -0.0580127, 0.2500000],
         [0.25000000, 0.4330127, -0.8660254],
         [-0.0580127, 0.89951905, 0.4330127]
     ]))
 
-    assert_array_almost_equal(dcm[2], np.array([
+    assert_array_almost_equal(mat[2], np.array([
         [0.65973961, -0.04736717, 0.7500000],
         [0.61237244, 0.61237244, -0.5000000],
         [-0.43559574, 0.78914913, 0.4330127]
@@ -437,21 +435,21 @@ def test_from_euler_extrinsic_rotation_313():
         [30, 60, 30],
         [45, 30, 60]
         ]
-    dcm = Rotation.from_euler('zxz', angles, degrees=True).as_dcm()
+    mat = Rotation.from_euler('zxz', angles, degrees=True).as_matrix()
 
-    assert_array_almost_equal(dcm[0], np.array([
+    assert_array_almost_equal(mat[0], np.array([
         [0.43559574, -0.65973961, 0.61237244],
         [0.78914913, -0.04736717, -0.61237244],
         [0.4330127, 0.75000000, 0.500000]
     ]))
 
-    assert_array_almost_equal(dcm[1], np.array([
+    assert_array_almost_equal(mat[1], np.array([
         [0.62500000, -0.64951905, 0.4330127],
         [0.64951905, 0.12500000, -0.750000],
         [0.4330127, 0.75000000, 0.500000]
     ]))
 
-    assert_array_almost_equal(dcm[2], np.array([
+    assert_array_almost_equal(mat[2], np.array([
         [-0.1767767, -0.88388348, 0.4330127],
         [0.91855865, -0.30618622, -0.250000],
         [0.35355339, 0.35355339, 0.8660254]
@@ -498,7 +496,8 @@ def test_as_euler_symmetric_axes():
 
 
 def test_as_euler_degenerate_asymmetric_axes():
-    # Since we cannot check for angle equality, we check for dcm equality
+    # Since we cannot check for angle equality, we check for rotation matrix
+    # equality
     angles = np.array([
         [45, 90, 35],
         [35, -90, 20],
@@ -511,30 +510,31 @@ def test_as_euler_degenerate_asymmetric_axes():
             # Extrinsic rotations
             seq = ''.join(seq_tuple)
             rotation = Rotation.from_euler(seq, angles, degrees=True)
-            dcm_expected = rotation.as_dcm()
+            mat_expected = rotation.as_matrix()
 
             angle_estimates = rotation.as_euler(seq, degrees=True)
-            dcm_estimated = Rotation.from_euler(
+            mat_estimated = Rotation.from_euler(
                 seq, angle_estimates, degrees=True
-                ).as_dcm()
+                ).as_matrix()
 
-            assert_array_almost_equal(dcm_expected, dcm_estimated)
+            assert_array_almost_equal(mat_expected, mat_estimated)
 
             # Intrinsic rotations
             seq = seq.upper()
             rotation = Rotation.from_euler(seq, angles, degrees=True)
-            dcm_expected = rotation.as_dcm()
+            mat_expected = rotation.as_matrix()
 
             angle_estimates = rotation.as_euler(seq, degrees=True)
-            dcm_estimated = Rotation.from_euler(
+            mat_estimated = Rotation.from_euler(
                 seq, angle_estimates, degrees=True
-                ).as_dcm()
+                ).as_matrix()
 
-            assert_array_almost_equal(dcm_expected, dcm_estimated)
+            assert_array_almost_equal(mat_expected, mat_estimated)
 
 
 def test_as_euler_degenerate_symmetric_axes():
-    # Since we cannot check for angle equality, we check for dcm equality
+    # Since we cannot check for angle equality, we check for rotation matrix
+    # equality
     angles = np.array([
         [15, 0, 60],
         [35, 0, 75],
@@ -551,26 +551,26 @@ def test_as_euler_degenerate_symmetric_axes():
                 # Extrinsic rotations
                 seq = axis1 + axis2 + axis1
                 rotation = Rotation.from_euler(seq, angles, degrees=True)
-                dcm_expected = rotation.as_dcm()
+                mat_expected = rotation.as_matrix()
 
                 angle_estimates = rotation.as_euler(seq, degrees=True)
-                dcm_estimated = Rotation.from_euler(
+                mat_estimated = Rotation.from_euler(
                     seq, angle_estimates, degrees=True
-                    ).as_dcm()
+                    ).as_matrix()
 
-                assert_array_almost_equal(dcm_expected, dcm_estimated)
+                assert_array_almost_equal(mat_expected, mat_estimated)
 
                 # Intrinsic rotations
                 seq = seq.upper()
                 rotation = Rotation.from_euler(seq, angles, degrees=True)
-                dcm_expected = rotation.as_dcm()
+                mat_expected = rotation.as_matrix()
 
                 angle_estimates = rotation.as_euler(seq, degrees=True)
-                dcm_estimated = Rotation.from_euler(
+                mat_estimated = Rotation.from_euler(
                     seq, angle_estimates, degrees=True
-                    ).as_dcm()
+                    ).as_matrix()
 
-                assert_array_almost_equal(dcm_expected, dcm_estimated)
+                assert_array_almost_equal(mat_expected, mat_estimated)
 
 
 def test_inv():
@@ -579,10 +579,10 @@ def test_inv():
     p = Rotation.from_quat(np.random.normal(size=(n, 4)))
     q = p.inv()
 
-    p_dcm = p.as_dcm()
-    q_dcm = q.as_dcm()
-    result1 = np.einsum('...ij,...jk->...ik', p_dcm, q_dcm)
-    result2 = np.einsum('...ij,...jk->...ik', q_dcm, p_dcm)
+    p_mat = p.as_matrix()
+    q_mat = q.as_matrix()
+    result1 = np.einsum('...ij,...jk->...ik', p_mat, q_mat)
+    result2 = np.einsum('...ij,...jk->...ik', q_mat, p_mat)
 
     eye3d = np.empty((n, 3, 3))
     eye3d[:] = np.eye(3)
@@ -596,10 +596,10 @@ def test_inv_single_rotation():
     p = Rotation.from_quat(np.random.normal(size=(4,)))
     q = p.inv()
 
-    p_dcm = p.as_dcm()
-    q_dcm = q.as_dcm()
-    res1 = np.dot(p_dcm, q_dcm)
-    res2 = np.dot(q_dcm, p_dcm)
+    p_mat = p.as_matrix()
+    q_mat = q.as_matrix()
+    res1 = np.dot(p_mat, q_mat)
+    res2 = np.dot(q_mat, p_mat)
 
     eye = np.eye(3)
 
@@ -609,10 +609,10 @@ def test_inv_single_rotation():
     x = Rotation.from_quat(np.random.normal(size=(1, 4)))
     y = x.inv()
 
-    x_dcm = x.as_dcm()
-    y_dcm = y.as_dcm()
-    result1 = np.einsum('...ij,...jk->...ik', x_dcm, y_dcm)
-    result2 = np.einsum('...ij,...jk->...ik', y_dcm, x_dcm)
+    x_matrix = x.as_matrix()
+    y_matrix = y.as_matrix()
+    result1 = np.einsum('...ij,...jk->...ik', x_matrix, y_matrix)
+    result2 = np.einsum('...ij,...jk->...ik', y_matrix, x_matrix)
 
     eye3d = np.empty((1, 3, 3))
     eye3d[:] = np.eye(3)
@@ -621,14 +621,126 @@ def test_inv_single_rotation():
     assert_array_almost_equal(result2, eye3d)
 
 
+def test_identity_magnitude():
+    n = 10
+    assert_allclose(Rotation.identity(n).magnitude(), 0)
+    assert_allclose(Rotation.identity(n).inv().magnitude(), 0)
+
+
+def test_single_identity_magnitude():
+    assert Rotation.identity().magnitude() == 0
+    assert Rotation.identity().inv().magnitude() == 0
+
+
+def test_identity_invariance():
+    n = 10
+    p = Rotation.random(n)
+    result = p * Rotation.identity(n) * p.inv()
+    assert_array_almost_equal(result.magnitude(), np.zeros(n))
+
+
+def test_single_identity_invariance():
+    n = 10
+    p = Rotation.random(n)
+    result = p * Rotation.identity() * p.inv()
+    assert_array_almost_equal(result.magnitude(), np.zeros(n))
+
+
+def test_magnitude():
+    r = Rotation.from_quat(np.eye(4))
+    result = r.magnitude()
+    assert_array_almost_equal(result, [np.pi, np.pi, np.pi, 0])
+
+    r = Rotation.from_quat(-np.eye(4))
+    result = r.magnitude()
+    assert_array_almost_equal(result, [np.pi, np.pi, np.pi, 0])
+
+
+def test_magnitude_single_rotation():
+    r = Rotation.from_quat(np.eye(4))
+    result1 = r[0].magnitude()
+    assert_allclose(result1, np.pi)
+
+    result2 = r[3].magnitude()
+    assert_allclose(result2, 0)
+
+
+def test_mean():
+    axes = np.concatenate((-np.eye(3), np.eye(3)))
+    thetas = np.linspace(0, np.pi / 2, 100)
+    for t in thetas:
+        r = Rotation.from_rotvec(t * axes)
+        assert_allclose(r.mean().magnitude(), 0, atol=1E-10)
+
+
+def test_weighted_mean():
+    # test that doubling a weight is equivalent to including a rotation twice.
+    axes = np.array([[0, 0, 0], [1, 0, 0], [1, 0, 0]])
+    thetas = np.linspace(0, np.pi / 2, 100)
+    for t in thetas:
+        rw = Rotation.from_rotvec(t * axes[:2])
+        mw = rw.mean(weights=[1, 2])
+
+        r = Rotation.from_rotvec(t * axes)
+        m = r.mean()
+        assert_allclose((m * mw.inv()).magnitude(), 0, atol=1E-10)
+
+
+def test_mean_invalid_weights():
+    with pytest.raises(ValueError, match="non-negative"):
+        r = Rotation.from_quat(np.eye(4))
+        r.mean(weights=-np.ones(4))
+
+
+def test_reduction_no_indices():
+    result = Rotation.identity().reduce(return_indices=False)
+    assert isinstance(result, Rotation)
+
+
+def test_reduction_none_indices():
+    result = Rotation.identity().reduce(return_indices=True)
+    assert type(result) == tuple
+    assert len(result) == 3
+
+    reduced, left_best, right_best = result
+    assert left_best is None
+    assert right_best is None
+
+
+def test_reduction_scalar_calculation():
+    rng = np.random.RandomState(0)
+    l = Rotation.random(5, random_state=rng)
+    r = Rotation.random(10, random_state=rng)
+    p = Rotation.random(7, random_state=rng)
+    reduced, left_best, right_best = p.reduce(l, r, return_indices=True)
+
+    # Loop implementation of the vectorized calculation in Rotation.reduce
+    scalars = np.zeros((len(l), len(p), len(r)))
+    for i, li in enumerate(l):
+        for j, pj in enumerate(p):
+            for k, rk in enumerate(r):
+                scalars[i, j, k] = np.abs((li * pj * rk).as_quat()[3])
+    scalars = np.reshape(np.rollaxis(scalars, 1), (scalars.shape[1], -1))
+
+    max_ind = np.argmax(np.reshape(scalars, (len(p), -1)), axis=1)
+    left_best_check = max_ind // len(r)
+    right_best_check = max_ind % len(r)
+    assert (left_best == left_best_check).all()
+    assert (right_best == right_best_check).all()
+
+    reduced_check = l[left_best_check] * p * r[right_best_check]
+    mag = (reduced.inv() * reduced_check).magnitude()
+    assert_array_almost_equal(mag, np.zeros(len(p)))
+
+
 def test_apply_single_rotation_single_point():
-    dcm = np.array([
+    mat = np.array([
         [0, -1, 0],
         [1, 0, 0],
         [0, 0, 1]
     ])
-    r_1d = Rotation.from_dcm(dcm)
-    r_2d = Rotation.from_dcm(np.expand_dims(dcm, axis=0))
+    r_1d = Rotation.from_matrix(mat)
+    r_2d = Rotation.from_matrix(np.expand_dims(mat, axis=0))
 
     v_1d = np.array([1, 2, 3])
     v_2d = np.expand_dims(v_1d, axis=0)
@@ -650,13 +762,13 @@ def test_apply_single_rotation_single_point():
 
 
 def test_apply_single_rotation_multiple_points():
-    dcm = np.array([
+    mat = np.array([
         [0, -1, 0],
         [1, 0, 0],
         [0, 0, 1]
     ])
-    r1 = Rotation.from_dcm(dcm)
-    r2 = Rotation.from_dcm(np.expand_dims(dcm, axis=0))
+    r1 = Rotation.from_matrix(mat)
+    r2 = Rotation.from_matrix(np.expand_dims(mat, axis=0))
 
     v = np.array([[1, 2, 3], [4, 5, 6]])
     v_rotated = np.array([[-2, 1, 3], [-5, 4, 6]])
@@ -671,18 +783,18 @@ def test_apply_single_rotation_multiple_points():
 
 
 def test_apply_multiple_rotations_single_point():
-    dcm = np.empty((2, 3, 3))
-    dcm[0] = np.array([
+    mat = np.empty((2, 3, 3))
+    mat[0] = np.array([
         [0, -1, 0],
         [1, 0, 0],
         [0, 0, 1]
     ])
-    dcm[1] = np.array([
+    mat[1] = np.array([
         [1, 0, 0],
         [0, 0, -1],
         [0, 1, 0]
     ])
-    r = Rotation.from_dcm(dcm)
+    r = Rotation.from_matrix(mat)
 
     v1 = np.array([1, 2, 3])
     v2 = np.expand_dims(v1, axis=0)
@@ -699,18 +811,18 @@ def test_apply_multiple_rotations_single_point():
 
 
 def test_apply_multiple_rotations_multiple_points():
-    dcm = np.empty((2, 3, 3))
-    dcm[0] = np.array([
+    mat = np.empty((2, 3, 3))
+    mat[0] = np.array([
         [0, -1, 0],
         [1, 0, 0],
         [0, 0, 1]
     ])
-    dcm[1] = np.array([
+    mat[1] = np.array([
         [1, 0, 0],
         [0, 0, -1],
         [0, 1, 0]
     ])
-    r = Rotation.from_dcm(dcm)
+    r = Rotation.from_matrix(mat)
 
     v = np.array([[1, 2, 3], [4, 5, 6]])
     v_rotated = np.array([[-2, 1, 3], [4, -6, 5]])
@@ -721,37 +833,37 @@ def test_apply_multiple_rotations_multiple_points():
 
 
 def test_getitem():
-    dcm = np.empty((2, 3, 3))
-    dcm[0] = np.array([
+    mat = np.empty((2, 3, 3))
+    mat[0] = np.array([
         [0, -1, 0],
         [1, 0, 0],
         [0, 0, 1]
     ])
-    dcm[1] = np.array([
+    mat[1] = np.array([
         [1, 0, 0],
         [0, 0, -1],
         [0, 1, 0]
     ])
-    r = Rotation.from_dcm(dcm)
+    r = Rotation.from_matrix(mat)
 
-    assert_allclose(r[0].as_dcm(), dcm[0])
-    assert_allclose(r[1].as_dcm(), dcm[1])
-    assert_allclose(r[:-1].as_dcm(), np.expand_dims(dcm[0], axis=0))
+    assert_allclose(r[0].as_matrix(), mat[0])
+    assert_allclose(r[1].as_matrix(), mat[1])
+    assert_allclose(r[:-1].as_matrix(), np.expand_dims(mat[0], axis=0))
 
 
 def test_n_rotations():
-    dcm = np.empty((2, 3, 3))
-    dcm[0] = np.array([
+    mat = np.empty((2, 3, 3))
+    mat[0] = np.array([
         [0, -1, 0],
         [1, 0, 0],
         [0, 0, 1]
     ])
-    dcm[1] = np.array([
+    mat[1] = np.array([
         [1, 0, 0],
         [0, 0, -1],
         [0, 1, 0]
     ])
-    r = Rotation.from_dcm(dcm)
+    r = Rotation.from_matrix(mat)
 
     assert_equal(len(r), 2)
     assert_equal(len(r[0]), 1)
@@ -766,32 +878,60 @@ def test_quat_ownership():
         [0, 1, 0, 0],
         [0, 0, 1, 0]
     ])
-    r = Rotation.from_quat(quat, normalized=True)
+    r = Rotation.from_quat(quat)
     s = r[0:2]
 
     r._quat[0] = np.array([0, -1, 0, 0])
     assert_allclose(s._quat[0], np.array([1, 0, 0, 0]))
 
 
-def test_match_vectors_no_rotation():
+def test_align_vectors_no_rotation():
     x = np.array([[1, 2, 3], [4, 5, 6]])
     y = x.copy()
 
-    r, p = Rotation.match_vectors(x, y)
-    assert_array_almost_equal(r.as_dcm(), np.eye(3))
+    r, rmsd = Rotation.align_vectors(x, y)
+    assert_array_almost_equal(r.as_matrix(), np.eye(3))
+    assert_allclose(rmsd, 0, atol=1e-6)
 
 
-def test_match_vectors_no_noise():
+def test_align_vectors_no_noise():
     np.random.seed(0)
     c = Rotation.from_quat(np.random.normal(size=4))
     b = np.random.normal(size=(5, 3))
     a = c.apply(b)
 
-    est, cov = Rotation.match_vectors(a, b)
+    est, rmsd = Rotation.align_vectors(a, b)
     assert_allclose(c.as_quat(), est.as_quat())
+    assert_allclose(rmsd, 0, atol=1e-7)
 
 
-def test_match_vectors_noise():
+def test_align_vectors_improper_rotation():
+    # Tests correct logic for issue #10444
+    x = np.array([[0.89299824, -0.44372674, 0.0752378],
+                  [0.60221789, -0.47564102, -0.6411702]])
+    y = np.array([[0.02386536, -0.82176463, 0.5693271],
+                  [-0.27654929, -0.95191427, -0.1318321]])
+
+    est, rmsd = Rotation.align_vectors(x, y)
+    assert_allclose(x, est.apply(y), atol=1e-6)
+    assert_allclose(rmsd, 0, atol=1e-7)
+
+
+def test_align_vectors_scaled_weights():
+    rng = np.random.RandomState(0)
+    c = Rotation.random(random_state=rng)
+    b = rng.normal(size=(5, 3))
+    a = c.apply(b)
+
+    est1, rmsd1, cov1 = Rotation.align_vectors(a, b, np.ones(5), True)
+    est2, rmsd2, cov2 = Rotation.align_vectors(a, b, 2 * np.ones(5), True)
+
+    assert_allclose(est1.as_matrix(), est2.as_matrix())
+    assert_allclose(np.sqrt(2) * rmsd1, rmsd2)
+    assert_allclose(cov1, cov2)
+
+
+def test_align_vectors_noise():
     np.random.seed(0)
     n_vectors = 100
     rot = Rotation.from_euler('xyz', np.random.normal(size=3))
@@ -812,7 +952,8 @@ def test_match_vectors_noise():
     # rotations to each vector.
     noisy_result = noise.apply(result)
 
-    est, cov = Rotation.match_vectors(noisy_result, vectors)
+    est, rmsd, cov = Rotation.align_vectors(noisy_result, vectors,
+                                            return_sensitivity=True)
 
     # Use rotation compositions to find out closeness
     error_vector = (rot * est.inv()).as_rotvec()
@@ -826,6 +967,34 @@ def test_match_vectors_noise():
     assert_allclose(cov[1, 1], 0, atol=tolerance)
     assert_allclose(cov[2, 2], 0, atol=tolerance)
 
+    assert_allclose(rmsd, np.sum((noisy_result - est.apply(vectors))**2)**0.5)
+
+
+def test_align_vectors_single_vector():
+    with pytest.warns(UserWarning, match="Optimal rotation is not"):
+        r_estimate, rmsd = Rotation.align_vectors([[1, -1, 1]], [[1, 1, -1]])
+        assert_allclose(rmsd, 0, atol=1e-16)
+
+
+def test_align_vectors_invalid_input():
+    with pytest.raises(ValueError, match="Expected input `a` to have shape"):
+        Rotation.align_vectors([1, 2, 3], [[1, 2, 3]])
+
+    with pytest.raises(ValueError, match="Expected input `b` to have shape"):
+        Rotation.align_vectors([[1, 2, 3]], [1, 2, 3])
+
+    with pytest.raises(ValueError, match="Expected inputs `a` and `b` "
+                                         "to have same shapes"):
+        Rotation.align_vectors([[1, 2, 3],[4, 5, 6]], [[1, 2, 3]])
+
+    with pytest.raises(ValueError,
+                       match="Expected `weights` to be 1 dimensional"):
+        Rotation.align_vectors([[1, 2, 3]], [[1, 2, 3]], weights=[[1]])
+
+    with pytest.raises(ValueError,
+                       match="Expected `weights` to have number of values"):
+        Rotation.align_vectors([[1, 2, 3]], [[1, 2, 3]], weights=[1, 2])
+
 
 def test_random_rotation_shape():
     assert_equal(Rotation.random().as_quat().shape, (4,))
@@ -927,7 +1096,7 @@ def test_slerp_call_time_dim_mismatch():
     s = Slerp(t, r)
 
     with pytest.raises(ValueError,
-                       match="times to be specified in a 1 dimensional array"):
+                       match="`times` must be at most 1-dimensional."):
         interp_times = np.array([[3.5],
                                  [4.2]])
         s(interp_times)
@@ -943,3 +1112,23 @@ def test_slerp_call_time_out_of_range():
         s([0, 1, 2])
     with pytest.raises(ValueError, match="times must be within the range"):
         s([1, 2, 6])
+
+
+def test_slerp_call_scalar_time():
+    r = Rotation.from_euler('X', [0, 80], degrees=True)
+    s = Slerp([0, 1], r)
+
+    r_interpolated = s(0.25)
+    r_interpolated_expected = Rotation.from_euler('X', 20, degrees=True)
+
+    delta = r_interpolated * r_interpolated_expected.inv()
+
+    assert_allclose(delta.magnitude(), 0, atol=1e-16)
+
+
+def test_multiplication_stability():
+    qs = Rotation.random(50, random_state=0)
+    rs = Rotation.random(1000, random_state=1)
+    for q in qs:
+        rs *= q * rs
+        assert_allclose(np.linalg.norm(rs.as_quat(), axis=1), 1)
