@@ -9,7 +9,7 @@
 #  Date: 2004-08-09
 #
 #  Modified: 2005-02-10 by Robert Kern.
-#              Contributed to Scipy
+#              Contributed to SciPy
 #            2005-10-07 by Robert Kern.
 #              Some fixes to match the new scipy_core
 #
@@ -17,24 +17,22 @@
 #
 #-------------------------------------------------------------------------------
 
-from __future__ import division, print_function, absolute_import
-
 # Standard library imports.
 import warnings
 
-# Scipy imports.
-from scipy._lib.six import callable, string_types
+# SciPy imports.
 from scipy import linalg, special
 from scipy.special import logsumexp
-from scipy._lib._numpy_compat import cov
+from scipy._lib._util import check_random_state
 
-from numpy import (atleast_2d, reshape, zeros, newaxis, dot, exp, pi, sqrt,
-                   ravel, power, atleast_1d, squeeze, sum, transpose, ones)
+from numpy import (asarray, atleast_2d, reshape, zeros, newaxis, dot, exp, pi,
+                   sqrt, ravel, power, atleast_1d, squeeze, sum, transpose,
+                   ones, cov)
 import numpy as np
-from numpy.random import choice, multivariate_normal
 
 # Local imports.
 from . import mvn
+from ._stats import gaussian_kernel_estimate
 
 
 __all__ = ['gaussian_kde']
@@ -190,7 +188,7 @@ class gaussian_kde(object):
 
     """
     def __init__(self, dataset, bw_method=None, weights=None):
-        self.dataset = atleast_2d(dataset)
+        self.dataset = atleast_2d(asarray(dataset))
         if not self.dataset.size > 1:
             raise ValueError("`dataset` input should have multiple elements.")
 
@@ -227,7 +225,7 @@ class gaussian_kde(object):
                      the dimensionality of the KDE.
 
         """
-        points = atleast_2d(points)
+        points = atleast_2d(asarray(points))
 
         d, m = points.shape
         if d != self.d:
@@ -240,28 +238,20 @@ class gaussian_kde(object):
                     self.d)
                 raise ValueError(msg)
 
-        result = zeros((m,), dtype=float)
-
-        whitening = linalg.cholesky(self.inv_cov)
-        scaled_dataset = dot(whitening, self.dataset)
-        scaled_points = dot(whitening, points)
-
-        if m >= self.n:
-            # there are more points than data, so loop over data
-            for i in range(self.n):
-                diff = scaled_dataset[:, i, newaxis] - scaled_points
-                energy = sum(diff * diff, axis=0) / 2.0
-                result += self.weights[i]*exp(-energy)
+        output_dtype = np.common_type(self.covariance, points)
+        itemsize = np.dtype(output_dtype).itemsize
+        if itemsize == 4:
+            spec = 'float'
+        elif itemsize == 8:
+            spec = 'double'
+        elif itemsize in (12, 16):
+            spec = 'long double'
         else:
-            # loop over points
-            for i in range(m):
-                diff = scaled_dataset - scaled_points[:, i, newaxis]
-                energy = sum(diff * diff, axis=0) / 2.0
-                result[i] = sum(exp(-energy)*self.weights, axis=0)
-
-        result = result * self.n / self._norm_factor
-
-        return result
+            raise TypeError('%s has unexpected item size %d' %
+                            (output_dtype, itemsize))
+        result = gaussian_kernel_estimate[spec](self.dataset.T, self.weights[:, None],
+                                                points.T, self.inv_cov, output_dtype)
+        return result[:, 0]
 
     __call__ = evaluate
 
@@ -436,7 +426,7 @@ class gaussian_kde(object):
 
         return result
 
-    def resample(self, size=None):
+    def resample(self, size=None, seed=None):
         """
         Randomly sample a dataset from the estimated pdf.
 
@@ -446,6 +436,16 @@ class gaussian_kde(object):
             The number of samples to draw.  If not provided, then the size is
             the same as the effective number of samples in the underlying
             dataset.
+        seed : {None, int, `~np.random.RandomState`, `~np.random.Generator`}, optional
+            This parameter defines the object to use for drawing random
+            variates.
+            If `seed` is `None` the `~np.random.RandomState` singleton is used.
+            If `seed` is an int, a new ``RandomState`` instance is used, seeded
+            with seed.
+            If `seed` is already a ``RandomState`` or ``Generator`` instance,
+            then that object is used.
+            Default is None.
+            Specify `seed` for reproducible drawing of random variates.
 
         Returns
         -------
@@ -456,17 +456,33 @@ class gaussian_kde(object):
         if size is None:
             size = int(self.neff)
 
-        norm = transpose(multivariate_normal(zeros((self.d,), float),
-                         self.covariance, size=size))
-        indices = choice(self.n, size=size, p=self.weights)
+        random_state = check_random_state(seed)
+        norm = transpose(random_state.multivariate_normal(
+            zeros((self.d,), float), self.covariance, size=size
+        ))
+        indices = random_state.choice(self.n, size=size, p=self.weights)
         means = self.dataset[:, indices]
 
         return means + norm
 
     def scotts_factor(self):
+        """Compute Scott's factor.
+
+        Returns
+        -------
+        s : float
+            Scott's factor.
+        """
         return power(self.neff, -1./(self.d+4))
 
     def silverman_factor(self):
+        """Compute the Silverman factor.
+
+        Returns
+        -------
+        s : float
+            The silverman factor.
+        """
         return power(self.neff*(self.d+2.0)/4.0, -1./(self.d+4))
 
     #  Default method to calculate bandwidth, can be overwritten by subclass
@@ -511,7 +527,7 @@ class gaussian_kde(object):
 
         >>> import matplotlib.pyplot as plt
         >>> fig, ax = plt.subplots()
-        >>> ax.plot(x1, np.ones(x1.shape) / (4. * x1.size), 'bo',
+        >>> ax.plot(x1, np.full(x1.shape, 1 / (4. * x1.size)), 'bo',
         ...         label='Data points (rescaled)')
         >>> ax.plot(xs, y1, label='Scott (default)')
         >>> ax.plot(xs, y2, label='Silverman')
@@ -526,7 +542,7 @@ class gaussian_kde(object):
             self.covariance_factor = self.scotts_factor
         elif bw_method == 'silverman':
             self.covariance_factor = self.silverman_factor
-        elif np.isscalar(bw_method) and not isinstance(bw_method, string_types):
+        elif np.isscalar(bw_method) and not isinstance(bw_method, str):
             self._bw_method = 'use constant'
             self.covariance_factor = lambda: bw_method
         elif callable(bw_method):
@@ -553,7 +569,7 @@ class gaussian_kde(object):
 
         self.covariance = self._data_covariance * self.factor**2
         self.inv_cov = self._data_inv_cov / self.factor**2
-        self._norm_factor = sqrt(linalg.det(2*pi*self.covariance)) * self.n
+        self._norm_factor = sqrt(linalg.det(2*pi*self.covariance))
 
     def pdf(self, x):
         """
@@ -585,8 +601,6 @@ class gaussian_kde(object):
                     self.d)
                 raise ValueError(msg)
 
-        result = zeros((m,), dtype=float)
-
         if m >= self.n:
             # there are more points than data, so loop over data
             energy = zeros((self.n, m), dtype=float)
@@ -594,17 +608,17 @@ class gaussian_kde(object):
                 diff = self.dataset[:, i, newaxis] - points
                 tdiff = dot(self.inv_cov, diff)
                 energy[i] = sum(diff*tdiff, axis=0) / 2.0
-            result = logsumexp(-energy,
-                               b=self.weights[i]*self.n/self._norm_factor,
-                               axis=0)
+            result = logsumexp(-energy.T,
+                               b=self.weights / self._norm_factor, axis=1)
         else:
             # loop over points
+            result = zeros((m,), dtype=float)
             for i in range(m):
                 diff = self.dataset - points[:, i, newaxis]
                 tdiff = dot(self.inv_cov, diff)
                 energy = sum(diff * tdiff, axis=0) / 2.0
-                result[i] = logsumexp(-energy,
-                                      b=self.weights*self.n/self._norm_factor)
+                result[i] = logsumexp(-energy, b=self.weights /
+                                      self._norm_factor)
 
         return result
 
